# validate.py
# A Validator object
# Copyright (C) 2005-2010 Michael Foord, Mark Andrews, Nicola Larosa
# E-mail: fuzzyman AT voidspace DOT org DOT uk
#         mark AT la-la DOT com
#         nico AT tekNico DOT net

# This software is licensed under the terms of the BSD license.
# http://www.voidspace.org.uk/python/license.shtml
# Basically you're free to copy, modify, distribute and relicense it,
# So long as you keep a copy of the license with it.

# Scripts maintained at http://www.voidspace.org.uk/python/index.shtml
# For information about bugfixes, updates and support, please join the
# ConfigObj mailing list:
# http://lists.sourceforge.net/lists/listinfo/configobj-develop
# Comments, suggestions and bug reports welcome.

# disable nose tests here
try:
    from nose.plugins.skip import SkipTest
except ImportError:
    pass
else:
    def setup_module():
        raise SkipTest('Doctests fail for externals.validator')

"""
    The Validator object is used to check that supplied values 
    conform to a specification.
    
    The value can be supplied as a string - e.g. from a config file.
    In this case the check will also *convert* the value to
    the required type. This allows you to add validation
    as a transparent layer to access data stored as strings.
    The validation checks that the data is correct *and*
    converts it to the expected type.
    
    Some standard checks are provided for basic data types.
    Additional checks are easy to write. They can be
    provided when the ``Validator`` is instantiated or
    added afterwards.
    
    The standard functions work with the following basic data types :
    
    * integers
    * floats
    * booleans
    * strings
    * ip_addr
    
    plus lists of these datatypes
    
    Adding additional checks is done through coding simple functions.
    
    The full set of standard checks are : 
    
    * 'integer': matches integer values (including negative)
                 Takes optional 'min' and 'max' arguments : ::
    
                   integer()
                   integer(3, 9)  # any value from 3 to 9
                   integer(min=0) # any positive value
                   integer(max=9)
    
    * 'float': matches float values
               Has the same parameters as the integer check.
    
    * 'boolean': matches boolean values - ``True`` or ``False``
                 Acceptable string values for True are :
                   true, on, yes, 1
                 Acceptable string values for False are :
                   false, off, no, 0
    
                 Any other value raises an error.
    
    * 'ip_addr': matches an Internet Protocol address, v.4, represented
                 by a dotted-quad string, i.e. '1.2.3.4'.
    
    * 'string': matches any string.
                Takes optional keyword args 'min' and 'max'
                to specify min and max lengths of the string.
    
    * 'list': matches any list.
              Takes optional keyword args 'min', and 'max' to specify min and
              max sizes of the list. (Always returns a list.)
    
    * 'tuple': matches any tuple.
              Takes optional keyword args 'min', and 'max' to specify min and
              max sizes of the tuple. (Always returns a tuple.)
    
    * 'int_list': Matches a list of integers.
                  Takes the same arguments as list.
    
    * 'float_list': Matches a list of floats.
                    Takes the same arguments as list.
    
    * 'bool_list': Matches a list of boolean values.
                   Takes the same arguments as list.
    
    * 'ip_addr_list': Matches a list of IP addresses.
                     Takes the same arguments as list.
    
    * 'string_list': Matches a list of strings.
                     Takes the same arguments as list.
    
    * 'mixed_list': Matches a list with different types in 
                    specific positions. List size must match
                    the number of arguments.
    
                    Each position can be one of :
                    'integer', 'float', 'ip_addr', 'string', 'boolean'
    
                    So to specify a list with two strings followed
                    by two integers, you write the check as : ::
    
                      mixed_list('string', 'string', 'integer', 'integer')
    
    * 'pass': This check matches everything ! It never fails
              and the value is unchanged.
    
              It is also the default if no check is specified.
    
    * 'option': This check matches any from a list of options.
                You specify this check with : ::
    
                  option('option 1', 'option 2', 'option 3')
    
    You can supply a default value (returned if no value is supplied)
    using the default keyword argument.
    
    You specify a list argument for default using a list constructor syntax in
    the check : ::
    
        checkname(arg1, arg2, default=list('val 1', 'val 2', 'val 3'))
    
    A badly formatted set of arguments will raise a ``VdtParamError``.
"""

__version__ = '1.0.1'


__all__ = (
    '__version__',
    'dottedQuadToNum',
    'numToDottedQuad',
    'ValidateError',
    'VdtUnknownCheckError',
    'VdtParamError',
    'VdtTypeError',
    'VdtValueError',
    'VdtValueTooSmallError',
    'VdtValueTooBigError',
    'VdtValueTooShortError',
    'VdtValueTooLongError',
    'VdtMissingValue',
    'Validator',
    'is_integer',
    'is_float',
    'is_boolean',
    'is_list',
    'is_tuple',
    'is_ip_addr',
    'is_string',
    'is_int_list',
    'is_bool_list',
    'is_float_list',
    'is_string_list',
    'is_ip_addr_list',
    'is_mixed_list',
    'is_option',
    '__docformat__',
)


import re


_list_arg = re.compile(r'''
    (?:
        ([a-zA-Z_][a-zA-Z0-9_]*)\s*=\s*list\(
            (
                (?:
                    \s*
                    (?:
                        (?:".*?")|              # double quotes
                        (?:'.*?')|              # single quotes
                        (?:[^'",\s\)][^,\)]*?)  # unquoted
                    )
                    \s*,\s*
                )*
                (?:
                    (?:".*?")|              # double quotes
                    (?:'.*?')|              # single quotes
                    (?:[^'",\s\)][^,\)]*?)  # unquoted
                )?                          # last one
            )
        \)
    )
''', re.VERBOSE | re.DOTALL)    # two groups

_list_members = re.compile(r'''
    (
        (?:".*?")|              # double quotes
        (?:'.*?')|              # single quotes
        (?:[^'",\s=][^,=]*?)       # unquoted
    )
    (?:
    (?:\s*,\s*)|(?:\s*$)            # comma
    )
''', re.VERBOSE | re.DOTALL)    # one group

_paramstring = r'''
    (?:
        (
            (?:
                [a-zA-Z_][a-zA-Z0-9_]*\s*=\s*list\(
                    (?:
                        \s*
                        (?:
                            (?:".*?")|              # double quotes
                            (?:'.*?')|              # single quotes
                            (?:[^'",\s\)][^,\)]*?)       # unquoted
                        )
                        \s*,\s*
                    )*
                    (?:
                        (?:".*?")|              # double quotes
                        (?:'.*?')|              # single quotes
                        (?:[^'",\s\)][^,\)]*?)       # unquoted
                    )?                              # last one
                \)
            )|
            (?:
                (?:".*?")|              # double quotes
                (?:'.*?')|              # single quotes
                (?:[^'",\s=][^,=]*?)|       # unquoted
                (?:                         # keyword argument
                    [a-zA-Z_][a-zA-Z0-9_]*\s*=\s*
                    (?:
                        (?:".*?")|              # double quotes
                        (?:'.*?')|              # single quotes
                        (?:[^'",\s=][^,=]*?)       # unquoted
                    )
                )
            )
        )
        (?:
            (?:\s*,\s*)|(?:\s*$)            # comma
        )
    )
    '''

_matchstring = '^%s*' % _paramstring

# Python pre 2.2.1 doesn't have bool
try:
    bool
except NameError:
    def bool(val):
        """Simple boolean equivalent function. """
        if val:
            return 1
        else:
            return 0


def dottedQuadToNum(ip):
    """
    Convert decimal dotted quad string to long integer
    
    >>> int(dottedQuadToNum('1 '))
    1
    >>> int(dottedQuadToNum(' 1.2'))
    16777218
    >>> int(dottedQuadToNum(' 1.2.3 '))
    16908291
    >>> int(dottedQuadToNum('1.2.3.4'))
    16909060
    >>> dottedQuadToNum('1.2.3. 4')
    16909060
    >>> dottedQuadToNum('255.255.255.255')
    4294967295L
    >>> dottedQuadToNum('255.255.255.256')
    Traceback (most recent call last):
    ValueError: Not a good dotted-quad IP: 255.255.255.256
    """
    
    # import here to avoid it when ip_addr values are not used
    import socket, struct
    
    try:
        return struct.unpack('!L',
            socket.inet_aton(ip.strip()))[0]
    except socket.error:
        # bug in inet_aton, corrected in Python 2.3
        if ip.strip() == '255.255.255.255':
            return 0xFFFFFFFFL
        else:
            raise ValueError('Not a good dotted-quad IP: %s' % ip)
    return


def numToDottedQuad(num):
    """
    Convert long int to dotted quad string
    
    >>> numToDottedQuad(-1L)
    Traceback (most recent call last):
    ValueError: Not a good numeric IP: -1
    >>> numToDottedQuad(1L)
    '0.0.0.1'
    >>> numToDottedQuad(16777218L)
    '1.0.0.2'
    >>> numToDottedQuad(16908291L)
    '1.2.0.3'
    >>> numToDottedQuad(16909060L)
    '1.2.3.4'
    >>> numToDottedQuad(4294967295L)
    '255.255.255.255'
    >>> numToDottedQuad(4294967296L)
    Traceback (most recent call last):
    ValueError: Not a good numeric IP: 4294967296
    """
    
    # import here to avoid it when ip_addr values are not used
    import socket, struct
    
    # no need to intercept here, 4294967295L is fine
    if num > 4294967295L or num < 0:
        raise ValueError('Not a good numeric IP: %s' % num)
    try:
        return socket.inet_ntoa(
            struct.pack('!L', long(num)))
    except (socket.error, struct.error, OverflowError):
        raise ValueError('Not a good numeric IP: %s' % num)


class ValidateError(Exception):
    """
    This error indicates that the check failed.
    It can be the base class for more specific errors.
    
    Any check function that fails ought to raise this error.
    (or a subclass)
    
    >>> raise ValidateError
    Traceback (most recent call last):
    ValidateError
    """


class VdtMissingValue(ValidateError):
    """No value was supplied to a check that needed one."""


class VdtUnknownCheckError(ValidateError):
    """An unknown check function was requested"""

    def __init__(self, value):
        """
        >>> raise VdtUnknownCheckError('yoda')
        Traceback (most recent call last):
        VdtUnknownCheckError: the check "yoda" is unknown.
        """
        ValidateError.__init__(self, 'the check "%s" is unknown.' % (value,))


class VdtParamError(SyntaxError):
    """An incorrect parameter was passed"""

    def __init__(self, name, value):
        """
        >>> raise VdtParamError('yoda', 'jedi')
        Traceback (most recent call last):
        VdtParamError: passed an incorrect value "jedi" for parameter "yoda".
        """
        SyntaxError.__init__(self, 'passed an incorrect value "%s" for parameter "%s".' % (value, name))


class VdtTypeError(ValidateError):
    """The value supplied was of the wrong type"""

    def __init__(self, value):
        """
        >>> raise VdtTypeError('jedi')
        Traceback (most recent call last):
        VdtTypeError: the value "jedi" is of the wrong type.
        """
        ValidateError.__init__(self, 'the value "%s" is of the wrong type.' % (value,))


class VdtValueError(ValidateError):
    """The value supplied was of the correct type, but was not an allowed value."""
    
    def __init__(self, value):
        """
        >>> raise VdtValueError('jedi')
        Traceback (most recent call last):
        VdtValueError: the value "jedi" is unacceptable.
        """
        ValidateError.__init__(self, 'the value "%s" is unacceptable.' % (value,))


class VdtValueTooSmallError(VdtValueError):
    """The value supplied was of the correct type, but was too small."""

    def __init__(self, value):
        """
        >>> raise VdtValueTooSmallError('0')
        Traceback (most recent call last):
        VdtValueTooSmallError: the value "0" is too small.
        """
        ValidateError.__init__(self, 'the value "%s" is too small.' % (value,))


class VdtValueTooBigError(VdtValueError):
    """The value supplied was of the correct type, but was too big."""

    def __init__(self, value):
        """
        >>> raise VdtValueTooBigError('1')
        Traceback (most recent call last):
        VdtValueTooBigError: the value "1" is too big.
        """
        ValidateError.__init__(self, 'the value "%s" is too big.' % (value,))


class VdtValueTooShortError(VdtValueError):
    """The value supplied was of the correct type, but was too short."""

    def __init__(self, value):
        """
        >>> raise VdtValueTooShortError('jed')
        Traceback (most recent call last):
        VdtValueTooShortError: the value "jed" is too short.
        """
        ValidateError.__init__(
            self,
            'the value "%s" is too short.' % (value,))


class VdtValueTooLongError(VdtValueError):
    """The value supplied was of the correct type, but was too long."""

    def __init__(self, value):
        """
        >>> raise VdtValueTooLongError('jedie')
        Traceback (most recent call last):
        VdtValueTooLongError: the value "jedie" is too long.
        """
        ValidateError.__init__(self, 'the value "%s" is too long.' % (value,))


class Validator(object):
    """
    Validator is an object that allows you to register a set of 'checks'.
    These checks take input and test that it conforms to the check.
    
    This can also involve converting the value from a string into
    the correct datatype.
    
    The ``check`` method takes an input string which configures which
    check is to be used and applies that check to a supplied value.
    
    An example input string would be:
    'int_range(param1, param2)'
    
    You would then provide something like:
    
    >>> def int_range_check(value, min, max):
    ...     # turn min and max from strings to integers
    ...     min = int(min)
    ...     max = int(max)
    ...     # check that value is of the correct type.
    ...     # possible valid inputs are integers or strings
    ...     # that represent integers
    ...     if not isinstance(value, (int, long, basestring)):
    ...         raise VdtTypeError(value)
    ...     elif isinstance(value, basestring):
    ...         # if we are given a string
    ...         # attempt to convert to an integer
    ...         try:
    ...             value = int(value)
    ...         except ValueError:
    ...             raise VdtValueError(value)
    ...     # check the value is between our constraints
    ...     if not min <= value:
    ...          raise VdtValueTooSmallError(value)
    ...     if not value <= max:
    ...          raise VdtValueTooBigError(value)
    ...     return value
    
    >>> fdict = {'int_range': int_range_check}
    >>> vtr1 = Validator(fdict)
    >>> vtr1.check('int_range(20, 40)', '30')
    30
    >>> vtr1.check('int_range(20, 40)', '60')
    Traceback (most recent call last):
    VdtValueTooBigError: the value "60" is too big.
    
    New functions can be added with : ::
    
    >>> vtr2 = Validator()       
    >>> vtr2.functions['int_range'] = int_range_check
    
    Or by passing in a dictionary of functions when Validator 
    is instantiated.
    
    Your functions *can* use keyword arguments,
    but the first argument should always be 'value'.
    
    If the function doesn't take additional arguments,
    the parentheses are optional in the check.
    It can be written with either of : ::
    
        keyword = function_name
        keyword = function_name()
    
    The first program to utilise Validator() was Michael Foord's
    ConfigObj, an alternative to ConfigParser which supports lists and
    can validate a config file using a config schema.
    For more details on using Validator with ConfigObj see:
    http://www.voidspace.org.uk/python/configobj.html
    """

    # this regex does the initial parsing of the checks
    _func_re = re.compile(r'(.+?)\((.*)\)', re.DOTALL)

    # this regex takes apart keyword arguments
    _key_arg = re.compile(r'^([a-zA-Z_][a-zA-Z0-9_]*)\s*=\s*(.*)$',  re.DOTALL)


    # this regex finds keyword=list(....) type values
    _list_arg = _list_arg

    # this regex takes individual values out of lists - in one pass
    _list_members = _list_members

    # These regexes check a set of arguments for validity
    # and then pull the members out
    _paramfinder = re.compile(_paramstring, re.VERBOSE | re.DOTALL)
    _matchfinder = re.compile(_matchstring, re.VERBOSE | re.DOTALL)


    def __init__(self, functions=None):
        """
        >>> vtri = Validator()
        """
        self.functions = {
            '': self._pass,
            'integer': is_integer,
            'float': is_float,
            'boolean': is_boolean,
            'ip_addr': is_ip_addr,
            'string': is_string,
            'list': is_list,
            'tuple': is_tuple,
            'int_list': is_int_list,
            'float_list': is_float_list,
            'bool_list': is_bool_list,
            'ip_addr_list': is_ip_addr_list,
            'string_list': is_string_list,
            'mixed_list': is_mixed_list,
            'pass': self._pass,
            'option': is_option,
            'force_list': force_list,
        }
        if functions is not None:
            self.functions.update(functions)
        # tekNico: for use by ConfigObj
        self.baseErrorClass = ValidateError
        self._cache = {}


    def check(self, check, value, missing=False):
        """
        Usage: check(check, value)
        
        Arguments:
            check: string representing check to apply (including arguments)
            value: object to be checked
        Returns value, converted to correct type if necessary
        
        If the check fails, raises a ``ValidateError`` subclass.
        
        >>> vtor.check('yoda', '')
        Traceback (most recent call last):
        VdtUnknownCheckError: the check "yoda" is unknown.
        >>> vtor.check('yoda()', '')
        Traceback (most recent call last):
        VdtUnknownCheckError: the check "yoda" is unknown.
        
        >>> vtor.check('string(default="")', '', missing=True)
        ''
        """
        fun_name, fun_args, fun_kwargs, default = self._parse_with_caching(check)
            
        if missing:
            if default is None:
                # no information needed here - to be handled by caller
                raise VdtMissingValue()
            value = self._handle_none(default)
                
        if value is None:
            return None
        
        return self._check_value(value, fun_name, fun_args, fun_kwargs)


    def _handle_none(self, value):
        if value == 'None':
            value = None
        elif value in ("'None'", '"None"'):
            # Special case a quoted None
            value = self._unquote(value)
        return value


    def _parse_with_caching(self, check):
        if check in self._cache:
            fun_name, fun_args, fun_kwargs, default = self._cache[check]
            # We call list and dict below to work with *copies* of the data
            # rather than the original (which are mutable of course)
            fun_args = list(fun_args)
            fun_kwargs = dict(fun_kwargs)
        else:
            fun_name, fun_args, fun_kwargs, default = self._parse_check(check)
            fun_kwargs = dict([(str(key), value) for (key, value) in fun_kwargs.items()])
            self._cache[check] = fun_name, list(fun_args), dict(fun_kwargs), default
        return fun_name, fun_args, fun_kwargs, default
        
        
    def _check_value(self, value, fun_name, fun_args, fun_kwargs):
        try:
            fun = self.functions[fun_name]
        except KeyError:
            raise VdtUnknownCheckError(fun_name)
        else:
            return fun(value, *fun_args, **fun_kwargs)


    def _parse_check(self, check):
        fun_match = self._func_re.match(check)
        if fun_match:
            fun_name = fun_match.group(1)
            arg_string = fun_match.group(2)
            arg_match = self._matchfinder.match(arg_string)
            if arg_match is None:
                # Bad syntax
                raise VdtParamError('Bad syntax in check "%s".' % check)
            fun_args = []
            fun_kwargs = {}
            # pull out args of group 2
            for arg in self._paramfinder.findall(arg_string):
                # args may need whitespace removing (before removing quotes)
                arg = arg.strip()
                listmatch = self._list_arg.match(arg)
                if listmatch:
                    key, val = self._list_handle(listmatch)
                    fun_kwargs[key] = val
                    continue
                keymatch = self._key_arg.match(arg)
                if keymatch:
                    val = keymatch.group(2)
                    if not val in ("'None'", '"None"'):
                        # Special case a quoted None
                        val = self._unquote(val)
                    fun_kwargs[keymatch.group(1)] = val
                    continue
                
                fun_args.append(self._unquote(arg))
        else:
            # allows for function names without (args)
            return check, (), {}, None

        # Default must be deleted if the value is specified too,
        # otherwise the check function will get a spurious "default" keyword arg
        try:
            default = fun_kwargs.pop('default', None)
        except AttributeError:
            # Python 2.2 compatibility
            default = None
            try:
                default = fun_kwargs['default']
                del fun_kwargs['default']
            except KeyError:
                pass
            
        return fun_name, fun_args, fun_kwargs, default


    def _unquote(self, val):
        """Unquote a value if necessary."""
        if (len(val) >= 2) and (val[0] in ("'", '"')) and (val[0] == val[-1]):
            val = val[1:-1]
        return val


    def _list_handle(self, listmatch):
        """Take apart a ``keyword=list('val, 'val')`` type string."""
        out = []
        name = listmatch.group(1)
        args = listmatch.group(2)
        for arg in self._list_members.findall(args):
            out.append(self._unquote(arg))
        return name, out


    def _pass(self, value):
        """
        Dummy check that always passes
        
        >>> vtor.check('', 0)
        0
        >>> vtor.check('', '0')
        '0'
        """
        return value
    
    
    def get_default_value(self, check):
        """
        Given a check, return the default value for the check
        (converted to the right type).
        
        If the check doesn't specify a default value then a
        ``KeyError`` will be raised.
        """
        fun_name, fun_args, fun_kwargs, default = self._parse_with_caching(check)
        if default is None:
            raise KeyError('Check "%s" has no default value.' % check)
        value = self._handle_none(default)
        if value is None:
            return value
        return self._check_value(value, fun_name, fun_args, fun_kwargs)


def _is_num_param(names, values, to_float=False):
    """
    Return numbers from inputs or raise VdtParamError.
    
    Lets ``None`` pass through.
    Pass in keyword argument ``to_float=True`` to
    use float for the conversion rather than int.
    
    >>> _is_num_param(('', ''), (0, 1.0))
    [0, 1]
    >>> _is_num_param(('', ''), (0, 1.0), to_float=True)
    [0.0, 1.0]
    >>> _is_num_param(('a'), ('a'))
    Traceback (most recent call last):
    VdtParamError: passed an incorrect value "a" for parameter "a".
    """
    fun = to_float and float or int
    out_params = []
    for (name, val) in zip(names, values):
        if val is None:
            out_params.append(val)
        elif isinstance(val, (int, long, float, basestring)):
            try:
                out_params.append(fun(val))
            except ValueError, e:
                raise VdtParamError(name, val)
        else:
            raise VdtParamError(name, val)
    return out_params


# built in checks
# you can override these by setting the appropriate name
# in Validator.functions
# note: if the params are specified wrongly in your input string,
#       you will also raise errors.

def is_integer(value, min=None, max=None):
    """
    A check that tests that a given value is an integer (int, or long)
    and optionally, between bounds. A negative value is accepted, while
    a float will fail.
    
    If the value is a string, then the conversion is done - if possible.
    Otherwise a VdtError is raised.
    
    >>> vtor.check('integer', '-1')
    -1
    >>> vtor.check('integer', '0')
    0
    >>> vtor.check('integer', 9)
    9
    >>> vtor.check('integer', 'a')
    Traceback (most recent call last):
    VdtTypeError: the value "a" is of the wrong type.
    >>> vtor.check('integer', '2.2')
    Traceback (most recent call last):
    VdtTypeError: the value "2.2" is of the wrong type.
    >>> vtor.check('integer(10)', '20')
    20
    >>> vtor.check('integer(max=20)', '15')
    15
    >>> vtor.check('integer(10)', '9')
    Traceback (most recent call last):
    VdtValueTooSmallError: the value "9" is too small.
    >>> vtor.check('integer(10)', 9)
    Traceback (most recent call last):
    VdtValueTooSmallError: the value "9" is too small.
    >>> vtor.check('integer(max=20)', '35')
    Traceback (most recent call last):
    VdtValueTooBigError: the value "35" is too big.
    >>> vtor.check('integer(max=20)', 35)
    Traceback (most recent call last):
    VdtValueTooBigError: the value "35" is too big.
    >>> vtor.check('integer(0, 9)', False)
    0
    """
    (min_val, max_val) = _is_num_param(('min', 'max'), (min, max))
    if not isinstance(value, (int, long, basestring)):
        raise VdtTypeError(value)
    if isinstance(value, basestring):
        # if it's a string - does it represent an integer ?
        try:
            value = int(value)
        except ValueError:
            raise VdtTypeError(value)
    if (min_val is not None) and (value < min_val):
        raise VdtValueTooSmallError(value)
    if (max_val is not None) and (value > max_val):
        raise VdtValueTooBigError(value)
    return value


def is_float(value, min=None, max=None):
    """
    A check that tests that a given value is a float
    (an integer will be accepted), and optionally - that it is between bounds.
    
    If the value is a string, then the conversion is done - if possible.
    Otherwise a VdtError is raised.
    
    This can accept negative values.
    
    >>> vtor.check('float', '2')
    2.0
    
    From now on we multiply the value to avoid comparing decimals
    
    >>> vtor.check('float', '-6.8') * 10
    -68.0
    >>> vtor.check('float', '12.2') * 10
    122.0
    >>> vtor.check('float', 8.4) * 10
    84.0
    >>> vtor.check('float', 'a')
    Traceback (most recent call last):
    VdtTypeError: the value "a" is of the wrong type.
    >>> vtor.check('float(10.1)', '10.2') * 10
    102.0
    >>> vtor.check('float(max=20.2)', '15.1') * 10
    151.0
    >>> vtor.check('float(10.0)', '9.0')
    Traceback (most recent call last):
    VdtValueTooSmallError: the value "9.0" is too small.
    >>> vtor.check('float(max=20.0)', '35.0')
    Traceback (most recent call last):
    VdtValueTooBigError: the value "35.0" is too big.
    """
    (min_val, max_val) = _is_num_param(
        ('min', 'max'), (min, max), to_float=True)
    if not isinstance(value, (int, long, float, basestring)):
        raise VdtTypeError(value)
    if not isinstance(value, float):
        # if it's a string - does it represent a float ?
        try:
            value = float(value)
        except ValueError:
            raise VdtTypeError(value)
    if (min_val is not None) and (value < min_val):
        raise VdtValueTooSmallError(value)
    if (max_val is not None) and (value > max_val):
        raise VdtValueTooBigError(value)
    return value


bool_dict = {
    True: True, 'on': True, '1': True, 'true': True, 'yes': True, 
    False: False, 'off': False, '0': False, 'false': False, 'no': False,
}


def is_boolean(value):
    """
    Check if the value represents a boolean.
    
    >>> vtor.check('boolean', 0)
    0
    >>> vtor.check('boolean', False)
    0
    >>> vtor.check('boolean', '0')
    0
    >>> vtor.check('boolean', 'off')
    0
    >>> vtor.check('boolean', 'false')
    0
    >>> vtor.check('boolean', 'no')
    0
    >>> vtor.check('boolean', 'nO')
    0
    >>> vtor.check('boolean', 'NO')
    0
    >>> vtor.check('boolean', 1)
    1
    >>> vtor.check('boolean', True)
    1
    >>> vtor.check('boolean', '1')
    1
    >>> vtor.check('boolean', 'on')
    1
    >>> vtor.check('boolean', 'true')
    1
    >>> vtor.check('boolean', 'yes')
    1
    >>> vtor.check('boolean', 'Yes')
    1
    >>> vtor.check('boolean', 'YES')
    1
    >>> vtor.check('boolean', '')
    Traceback (most recent call last):
    VdtTypeError: the value "" is of the wrong type.
    >>> vtor.check('boolean', 'up')
    Traceback (most recent call last):
    VdtTypeError: the value "up" is of the wrong type.
    
    """
    if isinstance(value, basestring):
        try:
            return bool_dict[value.lower()]
        except KeyError:
            raise VdtTypeError(value)
    # we do an equality test rather than an identity test
    # this ensures Python 2.2 compatibilty
    # and allows 0 and 1 to represent True and False
    if value == False:
        return False
    elif value == True:
        return True
    else:
        raise VdtTypeError(value)


def is_ip_addr(value):
    """
    Check that the supplied value is an Internet Protocol address, v.4,
    represented by a dotted-quad string, i.e. '1.2.3.4'.
    
    >>> vtor.check('ip_addr', '1 ')
    '1'
    >>> vtor.check('ip_addr', ' 1.2')
    '1.2'
    >>> vtor.check('ip_addr', ' 1.2.3 ')
    '1.2.3'
    >>> vtor.check('ip_addr', '1.2.3.4')
    '1.2.3.4'
    >>> vtor.check('ip_addr', '0.0.0.0')
    '0.0.0.0'
    >>> vtor.check('ip_addr', '255.255.255.255')
    '255.255.255.255'
    >>> vtor.check('ip_addr', '255.255.255.256')
    Traceback (most recent call last):
    VdtValueError: the value "255.255.255.256" is unacceptable.
    >>> vtor.check('ip_addr', '1.2.3.4.5')
    Traceback (most recent call last):
    VdtValueError: the value "1.2.3.4.5" is unacceptable.
    >>> vtor.check('ip_addr', 0)
    Traceback (most recent call last):
    VdtTypeError: the value "0" is of the wrong type.
    """
    if not isinstance(value, basestring):
        raise VdtTypeError(value)
    value = value.strip()
    try:
        dottedQuadToNum(value)
    except ValueError:
        raise VdtValueError(value)
    return value


def is_list(value, min=None, max=None):
    """
    Check that the value is a list of values.
    
    You can optionally specify the minimum and maximum number of members.
    
    It does no check on list members.
    
    >>> vtor.check('list', ())
    []
    >>> vtor.check('list', [])
    []
    >>> vtor.check('list', (1, 2))
    [1, 2]
    >>> vtor.check('list', [1, 2])
    [1, 2]
    >>> vtor.check('list(3)', (1, 2))
    Traceback (most recent call last):
    VdtValueTooShortError: the value "(1, 2)" is too short.
    >>> vtor.check('list(max=5)', (1, 2, 3, 4, 5, 6))
    Traceback (most recent call last):
    VdtValueTooLongError: the value "(1, 2, 3, 4, 5, 6)" is too long.
    >>> vtor.check('list(min=3, max=5)', (1, 2, 3, 4))
    [1, 2, 3, 4]
    >>> vtor.check('list', 0)
    Traceback (most recent call last):
    VdtTypeError: the value "0" is of the wrong type.
    >>> vtor.check('list', '12')
    Traceback (most recent call last):
    VdtTypeError: the value "12" is of the wrong type.
    """
    (min_len, max_len) = _is_num_param(('min', 'max'), (min, max))
    if isinstance(value, basestring):
        raise VdtTypeError(value)
    try:
        num_members = len(value)
    except TypeError:
        raise VdtTypeError(value)
    if min_len is not None and num_members < min_len:
        raise VdtValueTooShortError(value)
    if max_len is not None and num_members > max_len:
        raise VdtValueTooLongError(value)
    return list(value)


def is_tuple(value, min=None, max=None):
    """
    Check that the value is a tuple of values.
    
    You can optionally specify the minimum and maximum number of members.
    
    It does no check on members.
    
    >>> vtor.check('tuple', ())
    ()
    >>> vtor.check('tuple', [])
    ()
    >>> vtor.check('tuple', (1, 2))
    (1, 2)
    >>> vtor.check('tuple', [1, 2])
    (1, 2)
    >>> vtor.check('tuple(3)', (1, 2))
    Traceback (most recent call last):
    VdtValueTooShortError: the value "(1, 2)" is too short.
    >>> vtor.check('tuple(max=5)', (1, 2, 3, 4, 5, 6))
    Traceback (most recent call last):
    VdtValueTooLongError: the value "(1, 2, 3, 4, 5, 6)" is too long.
    >>> vtor.check('tuple(min=3, max=5)', (1, 2, 3, 4))
    (1, 2, 3, 4)
    >>> vtor.check('tuple', 0)
    Traceback (most recent call last):
    VdtTypeError: the value "0" is of the wrong type.
    >>> vtor.check('tuple', '12')
    Traceback (most recent call last):
    VdtTypeError: the value "12" is of the wrong type.
    """
    return tuple(is_list(value, min, max))


def is_string(value, min=None, max=None):
    """
    Check that the supplied value is a string.
    
    You can optionally specify the minimum and maximum number of members.
    
    >>> vtor.check('string', '0')
    '0'
    >>> vtor.check('string', 0)
    Traceback (most recent call last):
    VdtTypeError: the value "0" is of the wrong type.
    >>> vtor.check('string(2)', '12')
    '12'
    >>> vtor.check('string(2)', '1')
    Traceback (most recent call last):
    VdtValueTooShortError: the value "1" is too short.
    >>> vtor.check('string(min=2, max=3)', '123')
    '123'
    >>> vtor.check('string(min=2, max=3)', '1234')
    Traceback (most recent call last):
    VdtValueTooLongError: the value "1234" is too long.
    """
    if not isinstance(value, basestring):
        raise VdtTypeError(value)
    (min_len, max_len) = _is_num_param(('min', 'max'), (min, max))
    try:
        num_members = len(value)
    except TypeError:
        raise VdtTypeError(value)
    if min_len is not None and num_members < min_len:
        raise VdtValueTooShortError(value)
    if max_len is not None and num_members > max_len:
        raise VdtValueTooLongError(value)
    return value


def is_int_list(value, min=None, max=None):
    """
    Check that the value is a list of integers.
    
    You can optionally specify the minimum and maximum number of members.
    
    Each list member is checked that it is an integer.
    
    >>> vtor.check('int_list', ())
    []
    >>> vtor.check('int_list', [])
    []
    >>> vtor.check('int_list', (1, 2))
    [1, 2]
    >>> vtor.check('int_list', [1, 2])
    [1, 2]
    >>> vtor.check('int_list', [1, 'a'])
    Traceback (most recent call last):
    VdtTypeError: the value "a" is of the wrong type.
    """
    return [is_integer(mem) for mem in is_list(value, min, max)]


def is_bool_list(value, min=None, max=None):
    """
    Check that the value is a list of booleans.
    
    You can optionally specify the minimum and maximum number of members.
    
    Each list member is checked that it is a boolean.
    
    >>> vtor.check('bool_list', ())
    []
    >>> vtor.check('bool_list', [])
    []
    >>> check_res = vtor.check('bool_list', (True, False))
    >>> check_res == [True, False]
    1
    >>> check_res = vtor.check('bool_list', [True, False])
    >>> check_res == [True, False]
    1
    >>> vtor.check('bool_list', [True, 'a'])
    Traceback (most recent call last):
    VdtTypeError: the value "a" is of the wrong type.
    """
    return [is_boolean(mem) for mem in is_list(value, min, max)]


def is_float_list(value, min=None, max=None):
    """
    Check that the value is a list of floats.
    
    You can optionally specify the minimum and maximum number of members.
    
    Each list member is checked that it is a float.
    
    >>> vtor.check('float_list', ())
    []
    >>> vtor.check('float_list', [])
    []
    >>> vtor.check('float_list', (1, 2.0))
    [1.0, 2.0]
    >>> vtor.check('float_list', [1, 2.0])
    [1.0, 2.0]
    >>> vtor.check('float_list', [1, 'a'])
    Traceback (most recent call last):
    VdtTypeError: the value "a" is of the wrong type.
    """
    return [is_float(mem) for mem in is_list(value, min, max)]


def is_string_list(value, min=None, max=None):
    """
    Check that the value is a list of strings.
    
    You can optionally specify the minimum and maximum number of members.
    
    Each list member is checked that it is a string.
    
    >>> vtor.check('string_list', ())
    []
    >>> vtor.check('string_list', [])
    []
    >>> vtor.check('string_list', ('a', 'b'))
    ['a', 'b']
    >>> vtor.check('string_list', ['a', 1])
    Traceback (most recent call last):
    VdtTypeError: the value "1" is of the wrong type.
    >>> vtor.check('string_list', 'hello')
    Traceback (most recent call last):
    VdtTypeError: the value "hello" is of the wrong type.
    """
    if isinstance(value, basestring):
        raise VdtTypeError(value)
    return [is_string(mem) for mem in is_list(value, min, max)]


def is_ip_addr_list(value, min=None, max=None):
    """
    Check that the value is a list of IP addresses.
    
    You can optionally specify the minimum and maximum number of members.
    
    Each list member is checked that it is an IP address.
    
    >>> vtor.check('ip_addr_list', ())
    []
    >>> vtor.check('ip_addr_list', [])
    []
    >>> vtor.check('ip_addr_list', ('1.2.3.4', '5.6.7.8'))
    ['1.2.3.4', '5.6.7.8']
    >>> vtor.check('ip_addr_list', ['a'])
    Traceback (most recent call last):
    VdtValueError: the value "a" is unacceptable.
    """
    return [is_ip_addr(mem) for mem in is_list(value, min, max)]


def force_list(value, min=None, max=None):
    """
    Check that a value is a list, coercing strings into
    a list with one member. Useful where users forget the
    trailing comma that turns a single value into a list.
    
    You can optionally specify the minimum and maximum number of members.
    A minumum of greater than one will fail if the user only supplies a
    string.
    
    >>> vtor.check('force_list', ())
    []
    >>> vtor.check('force_list', [])
    []
    >>> vtor.check('force_list', 'hello')
    ['hello']
    """
    if not isinstance(value, (list, tuple)):
        value = [value]
    return is_list(value, min, max)
    
    

fun_dict = {
    'integer': is_integer,
    'float': is_float,
    'ip_addr': is_ip_addr,
    'string': is_string,
    'boolean': is_boolean,
}


def is_mixed_list(value, *args):
    """
    Check that the value is a list.
    Allow specifying the type of each member.
    Work on lists of specific lengths.
    
    You specify each member as a positional argument specifying type
    
    Each type should be one of the following strings :
      'integer', 'float', 'ip_addr', 'string', 'boolean'
    
    So you can specify a list of two strings, followed by
    two integers as :
    
      mixed_list('string', 'string', 'integer', 'integer')
    
    The length of the list must match the number of positional
    arguments you supply.
    
    >>> mix_str = "mixed_list('integer', 'float', 'ip_addr', 'string', 'boolean')"
    >>> check_res = vtor.check(mix_str, (1, 2.0, '1.2.3.4', 'a', True))
    >>> check_res == [1, 2.0, '1.2.3.4', 'a', True]
    1
    >>> check_res = vtor.check(mix_str, ('1', '2.0', '1.2.3.4', 'a', 'True'))
    >>> check_res == [1, 2.0, '1.2.3.4', 'a', True]
    1
    >>> vtor.check(mix_str, ('b', 2.0, '1.2.3.4', 'a', True))
    Traceback (most recent call last):
    VdtTypeError: the value "b" is of the wrong type.
    >>> vtor.check(mix_str, (1, 2.0, '1.2.3.4', 'a'))
    Traceback (most recent call last):
    VdtValueTooShortError: the value "(1, 2.0, '1.2.3.4', 'a')" is too short.
    >>> vtor.check(mix_str, (1, 2.0, '1.2.3.4', 'a', 1, 'b'))
    Traceback (most recent call last):
    VdtValueTooLongError: the value "(1, 2.0, '1.2.3.4', 'a', 1, 'b')" is too long.
    >>> vtor.check(mix_str, 0)
    Traceback (most recent call last):
    VdtTypeError: the value "0" is of the wrong type.
    
    This test requires an elaborate setup, because of a change in error string
    output from the interpreter between Python 2.2 and 2.3 .
    
    >>> res_seq = (
    ...     'passed an incorrect value "',
    ...     'yoda',
    ...     '" for parameter "mixed_list".',
    ... )
    >>> res_str = "'".join(res_seq)
    >>> try:
    ...     vtor.check('mixed_list("yoda")', ('a'))
    ... except VdtParamError, err:
    ...     str(err) == res_str
    1
    """
    try:
        length = len(value)
    except TypeError:
        raise VdtTypeError(value)
    if length < len(args):
        raise VdtValueTooShortError(value)
    elif length > len(args):
        raise VdtValueTooLongError(value)
    try:
        return [fun_dict[arg](val) for arg, val in zip(args, value)]
    except KeyError, e:
        raise VdtParamError('mixed_list', e)


def is_option(value, *options):
    """
    This check matches the value to any of a set of options.
    
    >>> vtor.check('option("yoda", "jedi")', 'yoda')
    'yoda'
    >>> vtor.check('option("yoda", "jedi")', 'jed')
    Traceback (most recent call last):
    VdtValueError: the value "jed" is unacceptable.
    >>> vtor.check('option("yoda", "jedi")', 0)
    Traceback (most recent call last):
    VdtTypeError: the value "0" is of the wrong type.
    """
    if not isinstance(value, basestring):
        raise VdtTypeError(value)
    if not value in options:
        raise VdtValueError(value)
    return value


def _test(value, *args, **keywargs):
    """
    A function that exists for test purposes.
    
    >>> checks = [
    ...     '3, 6, min=1, max=3, test=list(a, b, c)',
    ...     '3',
    ...     '3, 6',
    ...     '3,',
    ...     'min=1, test="a b c"',
    ...     'min=5, test="a, b, c"',
    ...     'min=1, max=3, test="a, b, c"',
    ...     'min=-100, test=-99',
    ...     'min=1, max=3',
    ...     '3, 6, test="36"',
    ...     '3, 6, test="a, b, c"',
    ...     '3, max=3, test=list("a", "b", "c")',
    ...     '''3, max=3, test=list("'a'", 'b', "x=(c)")''',
    ...     "test='x=fish(3)'",
    ...    ]
    >>> v = Validator({'test': _test})
    >>> for entry in checks:
    ...     print v.check(('test(%s)' % entry), 3)
    (3, ('3', '6'), {'test': ['a', 'b', 'c'], 'max': '3', 'min': '1'})
    (3, ('3',), {})
    (3, ('3', '6'), {})
    (3, ('3',), {})
    (3, (), {'test': 'a b c', 'min': '1'})
    (3, (), {'test': 'a, b, c', 'min': '5'})
    (3, (), {'test': 'a, b, c', 'max': '3', 'min': '1'})
    (3, (), {'test': '-99', 'min': '-100'})
    (3, (), {'max': '3', 'min': '1'})
    (3, ('3', '6'), {'test': '36'})
    (3, ('3', '6'), {'test': 'a, b, c'})
    (3, ('3',), {'test': ['a', 'b', 'c'], 'max': '3'})
    (3, ('3',), {'test': ["'a'", 'b', 'x=(c)'], 'max': '3'})
    (3, (), {'test': 'x=fish(3)'})
    
    >>> v = Validator()
    >>> v.check('integer(default=6)', '3')
    3
    >>> v.check('integer(default=6)', None, True)
    6
    >>> v.get_default_value('integer(default=6)')
    6
    >>> v.get_default_value('float(default=6)')
    6.0
    >>> v.get_default_value('pass(default=None)')
    >>> v.get_default_value("string(default='None')")
    'None'
    >>> v.get_default_value('pass')
    Traceback (most recent call last):
    KeyError: 'Check "pass" has no default value.'
    >>> v.get_default_value('pass(default=list(1, 2, 3, 4))')
    ['1', '2', '3', '4']
    
    >>> v = Validator()
    >>> v.check("pass(default=None)", None, True)
    >>> v.check("pass(default='None')", None, True)
    'None'
    >>> v.check('pass(default="None")', None, True)
    'None'
    >>> v.check('pass(default=list(1, 2, 3, 4))', None, True)
    ['1', '2', '3', '4']
    
    Bug test for unicode arguments
    >>> v = Validator()
    >>> v.check(u'string(min=4)', u'test')
    u'test'
    
    >>> v = Validator()
    >>> v.get_default_value(u'string(min=4, default="1234")')
    u'1234'
    >>> v.check(u'string(min=4, default="1234")', u'test')
    u'test'
    
    >>> v = Validator()
    >>> default = v.get_default_value('string(default=None)')
    >>> default == None
    1
    """
    return (value, args, keywargs)


def _test2():
    """
    >>> 
    >>> v = Validator()
    >>> v.get_default_value('string(default="#ff00dd")')
    '#ff00dd'
    >>> v.get_default_value('integer(default=3) # comment')
    3
    """

def _test3():
    r"""
    >>> vtor.check('string(default="")', '', missing=True)
    ''
    >>> vtor.check('string(default="\n")', '', missing=True)
    '\n'
    >>> print vtor.check('string(default="\n")', '', missing=True),
    <BLANKLINE>
    >>> vtor.check('string()', '\n')
    '\n'
    >>> vtor.check('string(default="\n\n\n")', '', missing=True)
    '\n\n\n'
    >>> vtor.check('string()', 'random \n text goes here\n\n')
    'random \n text goes here\n\n'
    >>> vtor.check('string(default=" \nrandom text\ngoes \n here\n\n ")',
    ... '', missing=True)
    ' \nrandom text\ngoes \n here\n\n '
    >>> vtor.check("string(default='\n\n\n')", '', missing=True)
    '\n\n\n'
    >>> vtor.check("option('\n','a','b',default='\n')", '', missing=True)
    '\n'
    >>> vtor.check("string_list()", ['foo', '\n', 'bar'])
    ['foo', '\n', 'bar']
    >>> vtor.check("string_list(default=list('\n'))", '', missing=True)
    ['\n']
    """
    
    
if __name__ == '__main__':
    # run the code tests in doctest format
    import sys
    import doctest
    m = sys.modules.get('__main__')
    globs = m.__dict__.copy()
    globs.update({
        'vtor': Validator(),
    })
    doctest.testmod(m, globs=globs)




from numpy.testing import assert_array_equal

from .exampledata import x, y
from .. import regression

# FIXME: This test does not test any values
# TODO: spend an hour or so to create a test like test_ols.py
# with R's output, the script and the data used for the script
#
# Although, it should be said that this, in R
# x = as.matrix(read_table('x.csv'))
# y = as.matrix(read_table('y.csv'))
# res = arima(y, xreg=x, order=c(2,0,0))
#
# gives an error ``system is computationally singular``

def test_armodel():
    for i in range(1,4):
        model = regression.ARModel(x, i)
        for i in range(20):
            results = model.fit(y)
            rho, sigma = regression.yule_walker(y - results.predicted)
            model = regression.ARModel(model.design, rho)
        print "AR coefficients:", model.rho




# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""
This module defines a class to output estimates
of delays and contrasts of delays.

Liao, C.H., Worsley, K.J., Poline, J-B., Aston, J.A.D., Duncan, G.H.,
Evans, A.C. (2002). \'Estimating the delay of the response in fMRI
data.\' NeuroImage, 16:593-606.

"""

__docformat__ = 'restructuredtext'

import os

import numpy as np
import numpy.linalg as L
from nipy.algorithms.utils.matrices import pos_recipr, recipr0
# FIXME: This is broken. Don't know how to fix it
#from nipy.fixes.scipy.stats.models.contrast import Contrast, ContrastResults


class Contrast(object):
    """ Empty boggus class to get the docs building.
    """
    # FIXME: This empty class needs to go.

# XXX FIXME This is broken

# class DelayContrast(Contrast):
#     """
#     Specify a delay contrast.

#     Delay contrasts are specified by a sequence of functions and weights, the
#     functions should NOT already be convolved with any HRF. They will be
#     convolved with self.IRF which is expected to be a filter with a canonical
#     HRF and its derivative -- defaults to the Glover model.

#     Weights should have the same number of columns as len(fns), with each row
#     specifying a different contrast.
#     """

#     Tmin = -100.
#     Tmax = 100.

#     def _sequence_call(self, time):
#         """
#         :Parameters:
#             `time` : TODO
#                 TODO

#         :Returns: ``numpy.ndarray``
#         """
#         return np.array([fn(time) for fn in self._sequence_fn])

#     def __init__(self, fns, weights, formula, IRF=None, name='', rownames=[]):
#         """
#         :Parameters:
#             `fns` : TODO
#                 TODO
#             `weights` : TODO
#                 TODO
#             `formula` : TODO
#                 TODO
#             `IRF` : TODO
#                 TODO
#             `name` : string
#                 TODO
#             `rownames` : [string]
#                 TODO
#         """
#         if IRF is None:
#             self.IRF = canonical
#         else:
#             self.IRF = IRF

#         self.delayflag = True

#         self.name = name
#         self.formula = formula
        
#         if type(fns) in [type([]), type(())]:
#             self._sequence_fn = fns
#             self._nsequence = len(_sequence_fn)
#             self.fn = self._sequence_call
#         else:
#             self.fn = fns

#         self.weights = np.asarray(weights)
#         if self.weights.ndim == 1:
#             self.weights.shape = (1, self.weights.shape[0])

# ##         if len(self._sequence_fn) != self.weights.shape[1]:
# ##             raise ValueError, 'length of weights does not match number of ' \
# ##                   'terms in DelayContrast'

#         term = ExperimentalQuantitative('%s_delay' % self.name, self.fn)
#         term.convolve(self.IRF)
        
#         Contrast.__init__(self, term, self.formula, name=self.name)

#         if rownames == []:
#             if name == '':
#                 raise ValueError, 'if rownames are not specified, name must be specified'
#             if self.weights.shape[0] > 1:
#                 self.rownames = ['%srow%d' % (name, i) for i in
#                                  range(self.weights.shape[0])]
#             elif self.weights.shape[0] == 1:
#                 self.rownames = ['']
#         else:
#             self.rownames = rownames

#     def compute_matrix(self, time=None):
#         """
#         :Parameters:
#             `time` : TODO
#                 TODO

#         :Returns: ``None``
#         """
#         Contrast.compute_matrix(self, time=time)

#         cnrow = self.matrix.shape[0] / 2
#         self.effectmatrix = self.matrix[0:cnrow]
#         self.deltamatrix = self.matrix[cnrow:]

#         self.isestimable(time)

#     def isestimable(self, t):
#         """
#         To estimate the delay, it is assumed that the response contains

#         (f ** HRF)(t + delta)

#         for each delay model time series 'f'.
#         More specifically, it is assumed that

#         f(t + delta) = c1 * (f ** HRF)(t) + delta * c2 * (f ** dHRF)(t)

#         where HRF and dHRF are the HRFs for this delay contrast.

#         This function checks to ensure that the columns

#         [(f ** HRF)(t), (f ** dHRF(t))]

#         are in the column space of the fMRI regression model.

#         :Parameters:
#             `t` : TODO
#                 TODO

#         :Returns: ``None``

#         :Raises ValueError: if any of the columns are not in the column space
#             of the model
#         """
        
#         D = self.formula(t).T
#         pinvD = L.pinv(D)

#         C = self.term(t)

#         cnrow = C.shape[0] / 2
#         effects = C[:cnrow]
#         deffects = C[cnrow:]

#         for i in range(self.weights.shape[0]):
#             for matrix in [effects, deffects]:
#                 col = np.dot(self.weights[i], matrix)
#                 colhat = np.dot(D, np.dot(pinvD, col))
#                 if not np.allclose(col, colhat):
#                     if self.weights.shape[0] > 1:
#                         name = self.rownames[i]
#                     else:
#                         name = ''
#                     raise ValueError, 'delay contrast %snot estimable' % name


#     def _extract_effect(self, results):

#         delay = self.IRF.delay

#         self.gamma0 = np.dot(self.effectmatrix, results.beta)
#         self.gamma1 = np.dot(self.deltamatrix, results.beta)

#         nrow = self.gamma0.shape[0]
#         self.T0sq = np.zeros(self.gamma0.shape)
        
#         for i in range(nrow):
#             self.T0sq[i] = (self.gamma0[i]**2 *
#                             recipr(results.cov_beta(matrix=self.effectmatrix[i])))

#         self.r = self.gamma1 * recipr0(self.gamma0)
#         self.rC = self.r * self.T0sq / (1. + self.T0sq)
#         self.deltahat = delay.inverse(self.rC)

#         self._effect = np.dot(self.weights, self.deltahat)

#     def _extract_sd(self, results):

#         delay = self.IRF.delay

#         self.T1 = np.zeros(self.gamma0.shape)

#         nrow = self.gamma0.shape[0]
#         for i in range(nrow):
#             self.T1[i] = self.gamma1[i] * recipr(np.sqrt(results.cov_beta(matrix=self.deltamatrix[i])))

#         a1 = 1 + 1. * recipr(self.T0sq)

#         gdot = np.array(([(self.r * (a1 - 2.) *
#                           recipr0(self.gamma0 * a1**2)),
#                          recipr0(self.gamma0 * a1)] *
#                         recipr0(delay.dforward(self.deltahat))))

#         Cov = results.cov_beta
#         E = self.effectmatrix
#         D = self.deltamatrix

#         nrow = self.effectmatrix.shape[0]
            
#         cov = np.zeros((nrow,)*2 + self.T0sq.shape[1:])

#         for i in range(nrow):
#             for j in range(i + 1):
#                 cov[i,j] = (gdot[0,i] * gdot[0,j] * Cov(matrix=E[i],
#                                                       other=E[j]) +  
#                             gdot[0,i] * gdot[1,j] * Cov(matrix=E[i],
#                                                       other=D[j]) +
#                             gdot[1,i] * gdot[0,j] * Cov(matrix=D[i],
#                                                       other=E[j]) +
#                             gdot[1,i] * gdot[1,j] * Cov(matrix=D[i],
#                                                       other=D[j]))
#                 cov[j,i] = cov[i,j]

#         nout = self.weights.shape[0]
#         self._sd = np.zeros(self._effect.shape)

#         for r in range(nout):
#             var = 0
#             for i in range(nrow):
#                 var += cov[i,i] * np.power(self.weights[r,i], 2)
#                 for j in range(i):
#                     var += 2 * cov[i,j] * self.weights[r,i] * self.weights[r,j]

#             self._sd[r] = np.sqrt(var)                

#     def _extract_t(self):
#         t = self._effect * recipr(self._sd)        
#         t = np.clip(t, self.Tmin, self.Tmax)
#         return t

#     def extract(self, results):
#         """
#         :Parameters:
#             `results` : TODO
#                 TODO
        
#         :Returns: `ContrastResults`
#         """
#         self._extract_effect(results)
#         self._extract_sd(results)
#         t = self._extract_t()

#         return ContrastResults(effect=self._effect,
#                                sd=self._sd,
#                                t=t, df_denom=results.df_resid)



# class DelayContrastOutput(TOutput):
#     """
#     TODO
#     """

#     def __init__(self, coordmap, contrast, IRF=None, dt=0.01, delta=None, 
#                  subpath='delays', clobber=False, path='.',
#                  ext='.hdr', volume_start_times=[], **kw):
#         """
#         :Parameters:
#             `coordmap` : TODO
#                 TODO
#             `contrast` : TODO
#                 TODO
#             `IRF` : TODO
#                 TODO
#             `dt` : float
#                 TODO
#             `delta` : TODO
#                 TODO
#             `subpath` : string
#                 TODO
#             `clobber` : bool
#                 TODO
#             `path` : string
#                 TODO
#             `ext` : string
#                 TODO
#             `volume_start_times` : TODO
#                 TODO
#             `kw` : dict
#                 Passed through to the constructor of `TContrastOutput`
            
#         """
#         TContrastOutput.__init__(self, coordmap, contrast, subpath=subpath,
#                                  clobber=clobber, volume_start_times=volume_start_times, **kw)
#         self.IRF = IRF
#         self.dt = dt
#         if delta is None:
#             self.delta = np.linspace(-4.5, 4.5, 91)
#         else:
#             self.delta = delta
#         self.path = path
#         self.subpath = subpath
#         self.clobber = clobber
#         self._setup_output_delay(path, clobber, subpath, ext, volume_start_times)

#     def _setup_contrast(self, time=None):
#         """
#         Setup the contrast for the delay.
#         """

#         self.contrast.compute_matrix(time=time)

#     def _setup_output_delay(self, path, clobber, subpath, ext, volume_start_times):
#         """
#         Setup the output for contrast, the DelayContrast. One t, sd, and
#         effect img is output for each row of contrast.weights. Further,
#         the \'magnitude\' (canonical HRF) contrast matrix and \'magnitude\'
#         column space are also output to illustrate what contrast this
#         corresponds to.

#         :Parameters:
#             `path` : string
#                 TODO
#             `clobber` : bool
#                 TODO
#             `subpath` : string
#                 TODO
#             `ext` : TODO
#                 TODO
#             `volume_start_times` : TODO
#                 TODO

#         :Returns: ``None``
#         """

#         self.timgs = []
#         self.sdimgs = []
#         self.effectimgs = []

#         self.timg_iters = []
#         self.sdimg_iters = []
#         self.effectimg_iters = []

#         nout = self.contrast.weights.shape[0]

#         for i in range(nout):
#             rowname = self.contrast.rownames[i]
#             outdir = os.path.join(path, subpath, rowname)
#             if not os.path.exists(outdir):
#                 os.makedirs(outdir)

#             cnrow = self.contrast.matrix.shape[0] / 2
#             l = np.zeros(self.contrast.matrix.shape[0])
#             l[0:cnrow] = self.contrast.weights[i]

#             img, it = self._setup_img(clobber, outdir, ext, "t")
#             self.timgs.append(img)
#             self.timg_iters.append(it)

#             img, it = self._setup_img(clobber, outdir, ext, "effect")
#             self.effectimgs.append(img)
#             self.effectimg_iters.append(it)

#             img, it = self._setup_img(clobber, outdir, ext, "sd")
#             self.sdimgs.append(img)
#             self.sdimg_iters.append(it)

#             matrix = np.squeeze(np.dot(l, self.contrast.matrix))

#             outname = os.path.join(outdir, 'matrix%s.csv' % rowname)
#             outfile = open(outname, 'w')
#             outfile.write(','.join(fpformat.fix(x,4) for x in matrix) + '\n')
#             outfile.close()

#             outname = os.path.join(outdir, 'matrix%s.bin' % rowname)
#             outfile = open(outname, 'w')
#             matrix = matrix.astype('<f8')
#             matrix.tofile(outfile)
#             outfile.close()


#             ftime = volume_start_times

#             def g(time=None, **extra):
#                 return np.squeeze(np.dot(l, self.contrast.term(time=time,
#                                                              **extra)))
#             f = pylab.gcf()
#             f.clf()
#             pl = MultiPlot(g, tmin=0, tmax=ftime.max(),
#                            dt = ftime.max() / 2000.,
#                            title='Magnitude column space for delay: \'%s\'' % rowname)
#             pl.draw()
#             pylab.savefig(os.path.join(outdir, 'matrix%s.png' % rowname))
#             f.clf()
#             del(f)
#             del(g)

#     def extract(self, results):
#         """
#         :Parameters:
#             `results` : TODO
#                 TODO

#         :Returns: TODO        
#         """
#         return self.contrast.extract(results)

#     def set_next(self, data):
#         """
#         :Parameters:
#             `data` : TODO
#                 TODO

#         :Returns: ``None``
#         """
#         nout = self.contrast.weights.shape[0]
#         for i in range(nout):
#             self.timg_iters[i].next().set(data.t[i])
#             if self.effect:
#                 self.effectimg_iters[i].next().set(data.effect[i])
#             if self.sd:
#                 self.sdimg_iters[i].next().set(data.sd[i])





import numpy as np
from nipy.algorithms.statistics import onesample
from scipy.stats import norm

from nipy.testing import assert_almost_equal


def test_estimate_varatio(p=1.0e-04, sigma2=1):
    # This is a random test, but is design to fail only rarely....
    ntrial = 300
    n = 10
    random = np.zeros(10)
    rsd = np.zeros(n)
    sd = np.multiply.outer(
        np.linspace(0,1,40),
        np.ones(ntrial)
        ) + np.ones((40,ntrial)) 

    for i in range(n):
        Y = np.random.standard_normal((40,ntrial)) * np.sqrt((sd**2 + sigma2))
        results = onesample.estimate_varatio(Y, sd)
        results = onesample.estimate_varatio(Y, sd)
        random[i] = results['random'].mean()
        rsd[i] = results['random'].std()

    # Compute the mean just to be sure it works

    W = 1. / (sd**2 + results['random'])
    mu = onesample.estimate_mean(Y, np.sqrt(sd**2 + results['random']))['effect']
    yield assert_almost_equal, mu, (W*Y).sum(0) / W.sum(0)

    rsd = np.sqrt((rsd**2).mean() / ntrial)
    T = np.fabs((random.mean() - sigma2) / (rsd / np.sqrt(n)))
    
    # should fail one in every 1/p trials at least for sigma > 0,
    # small values of sigma seem to have some bias
    if T > norm.ppf(1-p/2):
        raise ValueError('large T value, but algorithm works, '
                         'could be a statistical failure')



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
import os

def configuration(parent_package='',top_path=None):
    
    from numpy.distutils.misc_util import Configuration

    config = Configuration('segmentation', parent_package, top_path)
    config.add_subpackage('tests')
    config.add_include_dirs(config.name.replace('.', os.sep))
    config.add_extension('_segmentation', sources=['_segmentation.pyx', 
                                                   'mrf.c'])

    return config


if __name__ == '__main__':
    print('This is the wrong setup.py file to run')



""" Testing combined transformation objects

The combined transform object associates a spatial transformation with the
parameters of that transformation, for use in an optimizer.

The combined transform object does several things.  First, it can transform a
coordinate array with::

    transformed_pts = obj.apply(pts)

Second, the transform can phrase itself as a vector of parameters that are
suitable for optimization::

    vec = obj.get_params()

Third, the transform can be modified by setting from the optimization
parameters::

    obj.set_params(new_vec)
    new_transformed_pts = obj.apply(pts)

"""

import numpy as np
import numpy.linalg as npl

from nibabel.affines import apply_affine

from ..chain_transform import ChainTransform
from ..affine import Affine

from numpy.testing import (assert_array_almost_equal,
                           assert_array_equal)

from nose.tools import assert_true, assert_equal, assert_raises

AFF1 = np.diag([2, 3, 4, 1])
AFF2 = np.eye(4)
AFF2[:3,3] = (10, 11, 12)
# generate a random affine with a positive determinant
AFF3 = np.eye(4)
AFF3[:3,3] = np.random.normal(size=(3,))
tmp = np.random.normal(size=(3,3))
AFF3[:3,:3] = np.sign(npl.det(tmp))*tmp 
POINTS = np.arange(12).reshape(4,3)
# Make affine objects
AFF1_OBJ, AFF2_OBJ, AFF3_OBJ = [Affine(a) for a in [AFF1, AFF2.copy(), AFF3]]


def test_creation():
    # This is the simplest possible example, where there is a thing we are
    # optimizing, and an optional pre and post transform
    # Reset the aff2 object
    aff2_obj = Affine(AFF2.copy())
    ct = ChainTransform(aff2_obj)
    # Check apply gives expected result
    assert_array_equal(ct.apply(POINTS),
                       apply_affine(AFF2, POINTS))
    # Check that result is changed by setting params
    assert_array_equal(ct.param, aff2_obj.param)
    ct.param = np.zeros((12,))
    assert_array_almost_equal(ct.apply(POINTS), POINTS)
    # Does changing params in chain object change components passed in?
    assert_array_almost_equal(aff2_obj.param, np.zeros((12,)))
    # Reset the aff2 object
    aff2_obj = Affine(AFF2.copy())
    # Check apply gives the expected results
    ct = ChainTransform(aff2_obj, pre=AFF1_OBJ)
    assert_array_almost_equal(AFF1_OBJ.as_affine(), AFF1)
    assert_array_almost_equal(aff2_obj.as_affine(), AFF2)
    tmp = np.dot(AFF2, AFF1)
    assert_array_almost_equal(ct.apply(POINTS),
                       apply_affine(np.dot(AFF2, AFF1), POINTS))
    # Check that result is changed by setting params
    assert_array_almost_equal(ct.param, aff2_obj.param)
    ct.param = np.zeros((12,))
    assert_array_almost_equal(ct.apply(POINTS), apply_affine(AFF1, POINTS))
    # Does changing params in chain object change components passed in?
    assert_array_almost_equal(aff2_obj.param, np.zeros((12,)))
    # Reset the aff2 object
    aff2_obj = Affine(AFF2.copy())
    ct = ChainTransform(aff2_obj, pre=AFF1_OBJ, post=AFF3_OBJ)
    assert_array_almost_equal(ct.apply(POINTS),
                       apply_affine(np.dot(AFF3, np.dot(AFF2, AFF1)), POINTS))
    # Check that result is changed by setting params
    assert_array_equal(ct.param, aff2_obj.param)
    ct.param = np.zeros((12,))
    assert_array_almost_equal(ct.apply(POINTS),
                              apply_affine(np.dot(AFF3, AFF1), POINTS))
    # Does changing params in chain object change components passed in?
    assert_array_equal(aff2_obj.param, np.zeros((12,)))


# disabling this test because ChainTransform now returns an error if
# it doesn't get an optimizable transform.
"""
def test_inputs():
    # Check that we can pass arrays or None as pre and post
    assert_array_almost_equal(ChainTransform(AFF2).apply(POINTS),
                              ChainTransform(AFF2_OBJ).apply(POINTS))
    assert_array_almost_equal(ChainTransform(AFF2, pre=AFF1).apply(POINTS),
                              ChainTransform(AFF2_OBJ, pre=AFF1_OBJ).apply(POINTS))
    assert_array_almost_equal(ChainTransform(AFF2, pre=AFF1, post=AFF3).apply(POINTS),
                              ChainTransform(AFF2_OBJ, pre=AFF1_OBJ, post=AFF3_OBJ).apply(POINTS))
    assert_array_almost_equal(ChainTransform(AFF2, pre=None).apply(POINTS),
                              ChainTransform(AFF2_OBJ).apply(POINTS))
    assert_array_almost_equal(ChainTransform(AFF2, pre=None, post=None).apply(POINTS),
                              ChainTransform(AFF2_OBJ).apply(POINTS))
"""



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:

import numpy as np
import numpy.linalg as L
from scipy.stats import f as FDbn
from nipy.algorithms.statistics.models.regression import OLSModel, GLSModel

from nipy.core.api import Image
from nipy.modalities.fmri.fmristat import model as fmristat
from nipy.modalities.fmri.fmristat.model import OLS

from . import correlation, reml


def Fmask(Fimg, dfnum, dfdenom, pvalue=1.0e-04):
    """
    Create mask for use in estimating pooled covariance based on
    an F contrast.
    """

    ## TODO check nipy.algorithms.statistics.models.contrast to see if rank is
    ## correctly set -- I don't think it is right now.
    print dfnum, dfdenom
    thresh = FDbn.ppf(pvalue, dfnum, dfdenom)
    return Image(np.greater(np.asarray(Fimg), thresh), Fimg.grid.copy())


def estimate_pooled_covariance(resid, ARtarget=[0.3], mask=None):
    """
    Use SPM's REML implementation to estimate a pooled covariance matrix.
    
    Thresholds an F statistic at a marginal pvalue to estimate
    covariance matrix.

    """
    resid 
    n = resid[:].shape[0]
    components = correlation.ARcomponents(ARtarget, n)

    raw_sigma = 0
    nvox = 0
    for i in range(resid.shape[1]):
        d = np.asarray(resid[:,i])
        d.shape = (d.shape[0], np.product(d.shape[1:]))
        keep = np.asarray(mask[i])
        keep.shape = np.product(keep.shape)
        d = d.compress(keep, axis=1)
        raw_sigma += np.dot(d, d.T)
        nvox += d.shape[1]
    raw_sigma /= nvox
    C, h, _ = reml.reml(raw_sigma,
                        components,
                        n=nvox)
    return C

class SecondStage(object):
    """
    Parameters
    ----------
    fmri_image : `FmriImageList`
       object returning 4D array from ``np.asarray``, having attribute
       ``volume_start_times`` (if `volume_start_times` is None), and
       such that ``object[0]`` returns something with attributes ``shape``
    formula :  :class:`nipy.algorithms.statistics.formula.Formula`
    sigma : 
    outputs :
    volume_start_times : 
    """

    def __init__(self, fmri_image, formula, sigma, outputs=[],
                 volume_start_times=None):
        self.fmri_image = fmri_image
        self.data = np.asarray(fmri_image)
        self.formula = formula
        self.outputs = outputs
        self.sigma = sigma

        if volume_start_times is None:
            self.volume_start_times = self.fmri_image.volume_start_times
        else:
            self.volume_start_times = volume_start_times

    def execute(self):
        def model_params(*args):
            return (self.sigma,)
        m = fmristat.model_generator(self.formula, self.data,
                                     self.volume_start_times,
                                     model_type=GLSModel,
                                     model_params=model_params)
        r = fmristat.results_generator(m)

        def reshape(i, x):
            """
            To write output, arrays have to be reshaped --
            this function does the appropriate reshaping for the two
            passes of fMRIstat.

            These passes are i) 'slices through the z-axis'
                            ii) 'parcels of approximately constant AR1 coefficient'
            """
    
            if len(x.shape) == 2:
                if type(i) is type(1):
                    x.shape = (x.shape[0],) + self.fmri_image[0].shape[1:]                        
                if type(i) not in [type([]), type(())]:
                    i = (i,)
                else:
                    i = tuple(i)
                i = (slice(None,None,None),) + tuple(i)
            else:
                if type(i) is type(1):
                    x.shape = self.fmri_image[0].shape[1:]
            return i, x

        o = fmristat.generate_output(self.outputs, r, reshape=reshape)





# -*- Mode: Python -*-  Not really, but the syntax is close enough

"""
Miscellaneous fff routines.

Author: Alexis Roche, 2008.
"""

__version__ = '0.1'

# Includes
from fff cimport *
cimport numpy as cnp

# Exports from fff_gen_stats.h
cdef extern from "fff_gen_stats.h":

    double fff_mahalanobis(fff_vector* x, fff_matrix* S, fff_matrix* Saux)
    void fff_permutation(unsigned int* x, unsigned int n,
                         unsigned long int magic)
    void fff_combination(unsigned int* x, unsigned int k, unsigned int n,
                         unsigned long magic)

# Exports from fff_specfun.h
cdef extern from "fff_specfun.h":

    extern double fff_gamln(double x)
    extern double fff_psi(double x)

    
# Exports from fff_lapack.h
cdef extern from "fff_lapack.h":

    extern int fff_lapack_dgesdd(fff_matrix* A, fff_vector* s, fff_matrix* U, fff_matrix* Vt, 
                                 fff_vector* work, fff_array* iwork, fff_matrix* Aux)


# Initialize numpy
fffpy_import_array()
cnp.import_array()
import numpy as np

# This is faster than scipy.stats.scoreatpercentile due to partial
# sorting
def quantile(X, double ratio, int interp=False, int axis=0):
    """
    q = quantile(data, ratio, interp=False, axis=0).

    Partial sorting algorithm, very fast!!!
    """
    cdef fff_vector *x, *y
    cdef fffpy_multi_iterator* multi

    # Allocate output array Y
    dims = list(X.shape)
    dims[axis] = 1
    Y = np.zeros(dims)

    # Create a new array iterator 
    multi = fffpy_multi_iterator_new(2, axis, <void*>X, <void*>Y)

    # Create vector views on both X and Y 
    x = multi.vector[0]
    y = multi.vector[1]

    # Loop 
    while(multi.index < multi.size):
        y.data[0] = fff_vector_quantile(x, ratio, interp) 
        fffpy_multi_iterator_update(multi)
       
    # Delete local structures
    fffpy_multi_iterator_delete(multi)
    return Y

# This is faster than numpy.stats
# due to the underlying algorithm that relies on
# partial sorting as opposed to full sorting.
def median(x, axis=0):
    """
    median(x, axis=0).
    Equivalent to: quantile(x, ratio=0.5, interp=True, axis=axis).
    """
    return quantile(x, axis=axis, ratio=0.5, interp=True)


def mahalanobis(X, VX):
    """
    d2 = mahalanobis(X, VX).

    ufunc-like function to compute Mahalanobis squared distances
    x'*inv(Vx)*x.  

    axis == 0 assumed. If X is shaped (d,K), VX must be shaped
    (d,d,K).
    """
    cdef fff_vector *x, *vx, *x_tmp, *vx_tmp, *d2
    cdef fff_matrix Sx 
    cdef fff_matrix *Sx_tmp
    cdef fffpy_multi_iterator* multi
    cdef int axis=0, n

    # Allocate output array
    dims = list(X.shape)
    dim = dims[0]
    dims[0] = 1
    D2 = np.zeros(dims)

    # Flatten input variance array
    VX_flat = VX.reshape( [dim*dim]+list(VX.shape[2:]) )

    # Create a new array iterator 
    multi = fffpy_multi_iterator_new(3, axis, <void*>X, <void*>VX_flat, <void*>D2)

    # Allocate local structures 
    n = <int> X.shape[axis]
    x_tmp = fff_vector_new(n)
    vx_tmp = fff_vector_new(n*n) 
    Sx_tmp = fff_matrix_new(n, n)

    # Create vector views on X, VX_flat and D2 
    x = multi.vector[0]
    vx = multi.vector[1] 
    d2 = multi.vector[2] 

    # Loop 
    while(multi.index < multi.size):
        fff_vector_memcpy(x_tmp, x)
        fff_vector_memcpy(vx_tmp, vx) 
        Sx = fff_matrix_view(vx_tmp.data, n, n, n) # OK because vx_tmp is contiguous  
        d2.data[0] = fff_mahalanobis(x_tmp, &Sx, Sx_tmp)
        fffpy_multi_iterator_update(multi)

    # Delete local structs and views
    fff_vector_delete(x_tmp)
    fff_vector_delete(vx_tmp) 
    fff_matrix_delete(Sx_tmp)
    fffpy_multi_iterator_delete(multi)

    # Return
    D2 = D2.reshape(VX.shape[2:])
    return D2 


def svd(X):
    """ Singular value decomposition of array `X`

    Y = svd(X)

    ufunc-like svd. Given an array X (m, n, K), perform an SV decomposition.

    Parameters
    ----------
    X : 2D array

    Returns
    -------
    S : (min(m,n), K)
    """
    cdef int axis=0
    cdef int m, n, dmin, dmax, lwork, liwork, info
    cdef fff_vector *work, *x_flat, *x_flat_tmp, *s, *s_tmp
    cdef fff_matrix x
    cdef fff_array *iwork
    cdef fff_matrix *Aux, *U, *Vt 
    cdef fffpy_multi_iterator* multi

    # Shape of matrices
    m = <int> X.shape[0]
    n = <int> X.shape[1]
    if m > n:
        dmin = n
        dmax = m
    else:
        dmin = m
        dmax = n

    # Create auxiliary arrays
    lwork = 4*dmin*(dmin+1)
    if dmax > lwork:
        lwork = dmax
    lwork = 2*(3*dmin*dmin + lwork)
    liwork = 8*dmin
    work = fff_vector_new(lwork)
    iwork = fff_array_new1d(FFF_INT, liwork)
    Aux = fff_matrix_new(dmax, dmax)
    U = fff_matrix_new(m, m)
    Vt = fff_matrix_new(n, n)
    x_flat_tmp = fff_vector_new(m*n)
    s_tmp = fff_vector_new(dmin)

    # Allocate output array
    endims = list(X.shape[2:])
    S = np.zeros([dmin]+endims)

    # Flatten input array
    X_flat = X.reshape([m*n]+endims)

    # Create a new array iterator
    multi = fffpy_multi_iterator_new(2, axis, <void*>X_flat, <void*>S)

    # Create vector views
    x_flat = multi.vector[0]
    s = multi.vector[1]

    # Loop
    while(multi.index < multi.size):
        fff_vector_memcpy(x_flat_tmp, x_flat)
        fff_vector_memcpy(s_tmp, s)
        x = fff_matrix_view(x_flat_tmp.data, m, n, n) # OK because x_flat_tmp is contiguous
        info = fff_lapack_dgesdd(&x, s_tmp, U, Vt, work, iwork, Aux )
        fff_vector_memcpy(s, s_tmp)
        fffpy_multi_iterator_update(multi)

    # Delete local structures
    fff_vector_delete(work)
    fff_vector_delete(x_flat_tmp)
    fff_vector_delete(s_tmp)
    fff_array_delete(iwork)
    fff_matrix_delete(Aux)
    fff_matrix_delete(U)
    fff_matrix_delete(Vt)
    fffpy_multi_iterator_delete(multi)

    # Return
    return S


def permutations(unsigned int n, unsigned int m=1, unsigned long magic=0):
    """
    P = permutations(n, m=1, magic=0).
    Generate m permutations from [0..n[.
    """
    cdef fff_array *p, *pi
    cdef fff_array pi_view
    cdef unsigned int i
    p = fff_array_new2d(FFF_UINT, n, m)
    pi = fff_array_new1d(FFF_UINT, n) ## contiguous, dims=(n,1,1,1)

    for i from 0 <= i < m:
        fff_permutation(<unsigned int*>pi.data, n, magic+i)
        pi_view = fff_array_get_block2d(p, 0, n-1, 1, i, i, 1) ## dims=(n,1,1,1)
        fff_array_copy(&pi_view, pi)

    P = fff_array_toPyArray(p)
    return P 


def combinations(unsigned int k, unsigned int n, unsigned int m=1, unsigned long magic=0):
    """
    P = combinations(k, n, m=1, magic=0).
    Generate m combinations of k elements  from [0..n[.
    """
    cdef fff_array *p, *pi
    cdef fff_array pi_view
    cdef unsigned int i
    p = fff_array_new2d(FFF_UINT, k, m)
    pi = fff_array_new1d(FFF_UINT, k) ## contiguous, dims=(n,1,1,1)

    for i from 0 <= i < m:
        fff_combination(<unsigned int*>pi.data, k, n, magic+i)
        pi_view = fff_array_get_block2d(p, 0, k-1, 1, i, i, 1) ## dims=(k,1,1,1)
        fff_array_copy(&pi_view, pi)

    C = fff_array_toPyArray(p)
    return C


def gamln(double x):
    """ Python bindings to log gamma. Do not use, this is there only for
        testing. Use scipy.special.gammaln.
    """
    cdef double y
    y = fff_gamln(x)
    return y 


def psi(double x):
    """ Python bindings to psi (d gamln(x)/dx. Do not use, this is there only 
        for testing. Use scipy.special.psi.
    """
    cdef double y
    y = fff_psi(x)
    return y




#!/usr/bin/env python
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
from __future__ import print_function # Python 2/3 compatibility
__doc__ = """
Full step-by-step example of fitting a GLM to experimental data and visualizing
the results.

More specifically:

1. A sequence of fMRI volumes are loaded
2. A design matrix describing all the effects related to the data is computed
3. a mask of the useful brain volume is computed
4. A GLM is applied to the dataset (effect/covariance,
   then contrast estimation)

Note that this corresponds to a single run.

Needs matplotlib

Author : Bertrand Thirion, 2010--2012
"""
print(__doc__)

from os import mkdir, getcwd, path

import numpy as np

try:
    import matplotlib.pyplot as plt
except ImportError:
    raise RuntimeError("This script needs the matplotlib library")

from nibabel import save

from nipy.modalities.fmri.glm import FMRILinearModel
from nipy.modalities.fmri.design_matrix import make_dmtx
from nipy.modalities.fmri.experimental_paradigm import \
    load_paradigm_from_csv_file
from nipy.labs.viz import plot_map, cm

# Local import
from get_data_light import DATA_DIR, get_first_level_dataset

#######################################
# Data and analysis parameters
#######################################

# volume mask
# This dataset is large
get_first_level_dataset()
data_path = path.join(DATA_DIR, 's12069_swaloc1_corr.nii.gz')
paradigm_file = path.join(DATA_DIR, 'localizer_paradigm.csv')

# timing
n_scans = 128
tr = 2.4

# paradigm
frametimes = np.linspace(0, (n_scans - 1) * tr, n_scans)

# confounds
hrf_model = 'canonical with derivative'
drift_model = "cosine"
hfcut = 128

# write directory
write_dir = path.join(getcwd(), 'results')
if not path.exists(write_dir):
    mkdir(write_dir)

print('Computation will be performed in directory: %s' % write_dir)

########################################
# Design matrix
########################################

print('Loading design matrix...')

paradigm = load_paradigm_from_csv_file(paradigm_file)['0']

design_matrix = make_dmtx(frametimes, paradigm, hrf_model=hrf_model,
                          drift_model=drift_model, hfcut=hfcut)

ax = design_matrix.show()
ax.set_position([.05, .25, .9, .65])
ax.set_title('Design matrix')

plt.savefig(path.join(write_dir, 'design_matrix.png'))

#########################################
# Specify the contrasts
#########################################

# simplest ones
contrasts = {}
n_columns = len(design_matrix.names)
for i in range(paradigm.n_conditions):
    contrasts['%s' % design_matrix.names[2 * i]] = np.eye(n_columns)[2 * i]

# and more complex/ interesting ones
contrasts["audio"] = contrasts["clicDaudio"] + contrasts["clicGaudio"] +\
                     contrasts["calculaudio"] + contrasts["phraseaudio"]
contrasts["video"] = contrasts["clicDvideo"] + contrasts["clicGvideo"] + \
                     contrasts["calculvideo"] + contrasts["phrasevideo"]
contrasts["left"] = contrasts["clicGaudio"] + contrasts["clicGvideo"]
contrasts["right"] = contrasts["clicDaudio"] + contrasts["clicDvideo"]
contrasts["computation"] = contrasts["calculaudio"] + contrasts["calculvideo"]
contrasts["sentences"] = contrasts["phraseaudio"] + contrasts["phrasevideo"]
contrasts["H-V"] = contrasts["damier_H"] - contrasts["damier_V"]
contrasts["V-H"] = contrasts["damier_V"] - contrasts["damier_H"]
contrasts["left-right"] = contrasts["left"] - contrasts["right"]
contrasts["right-left"] = contrasts["right"] - contrasts["left"]
contrasts["audio-video"] = contrasts["audio"] - contrasts["video"]
contrasts["video-audio"] = contrasts["video"] - contrasts["audio"]
contrasts["computation-sentences"] = contrasts["computation"] -  \
                                     contrasts["sentences"]
contrasts["reading-visual"] = contrasts["sentences"] * 2 - \
                              contrasts["damier_H"] - contrasts["damier_V"]
contrasts['effects_of_interest'] = np.eye(25)[:20:2]

########################################
# Perform a GLM analysis
########################################

print('Fitting a GLM (this takes time)...')
fmri_glm = FMRILinearModel(data_path, design_matrix.matrix,
                           mask='compute')
fmri_glm.fit(do_scaling=True, model='ar1')

#########################################
# Estimate the contrasts
#########################################

print('Computing contrasts...')
for index, (contrast_id, contrast_val) in enumerate(contrasts.items()):
    print('  Contrast % 2i out of %i: %s' %
          (index + 1, len(contrasts), contrast_id))
    # save the z_image
    image_path = path.join(write_dir, '%s_z_map.nii' % contrast_id)
    z_map, = fmri_glm.contrast(contrast_val, con_id=contrast_id, output_z=True)
    save(z_map, image_path)

    # Create snapshots of the contrasts
    vmax = max(- z_map.get_data().min(), z_map.get_data().max())
    if index > 0:
        plt.clf()
    plot_map(z_map.get_data(), z_map.get_affine(),
             cmap=cm.cold_hot,
             vmin=- vmax,
             vmax=vmax,
             anat=None,
             figure=10,
             threshold=2.5)
    plt.savefig(path.join(write_dir, '%s_z_map.png' % contrast_id))

print("All the  results were witten in %s" % write_dir)

plt.show()



#!/usr/bin/env python
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""Create a nifti image from a numpy array and an affine transform.
"""

import numpy as np

from nipy import save_image, load_image
from nipy.core.api import Image, vox2scanner

# This gets the filename for a tiny example file
from nipy.testing import anatfile

# Load an image to get an array and affine
#
# Use one of our test files to get an array and affine (as numpy array) from.
img = load_image(anatfile)
arr = img.get_data()
affine_array = img.coordmap.affine.copy()

# 1) Create a CoordinateMap from the affine transform which specifies
# the mapping from input to output coordinates. The ``vox2scanner`` function
# makes a coordinate map from voxels to scanner coordinates.  Other options are
# ``vox2mni`` or ``vox2talairach``
affine_coordmap = vox2scanner(affine_array)

# 2) Create a nipy image from the array and CoordinateMap
newimg = Image(arr, affine_coordmap)

# Save the nipy image to the specified filename
save_image(newimg, 'an_image.nii.gz')

# Reload and verify the data and affine were saved correctly.
img_back = load_image('an_image.nii.gz')
assert np.allclose(img_back.get_data(), img.get_data())
assert np.allclose(img_back.coordmap.affine, img.coordmap.affine)



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
# add-ons to scipy.optimize

import numpy as np 
from scipy.optimize import brent, approx_fprime



def _linesearch_brent(func, p, xi, tol=1e-3):
    """Line-search algorithm using Brent's method.

    Find the minimium of the function ``func(x0+ alpha*direc)``.
    """
    def myfunc(alpha):
        return func(p + alpha * xi)
    alpha_min, fret, iter, num = brent(myfunc, full_output=1, tol=tol)
    xi = alpha_min*xi
    return np.squeeze(fret), p+xi


def _wrap(function, args):
    ncalls = [0]
    def wrapper(x):
        ncalls[0] += 1
        return function(x, *args)
    return ncalls, wrapper


def fmin_steepest(f, x0, fprime=None, xtol=1e-4, ftol=1e-4, 
                  maxiter=None, callback=None, disp=True):
    """
    Minimize a function using a steepest gradient descent
    algorithm. This complements the collection of minimization
    routines provided in scipy.optimize. Steepest gradient iterations
    are cheaper than in the conjugate gradient or Newton methods,
    hence convergence may sometimes turn out faster algthough more
    iterations are typically needed.

    Parameters
    ----------
    f : callable
      Function to be minimized
    x0 : array
      Starting point
    fprime : callable
      Function that computes the gradient of f
    xtol : float
      Relative tolerance on step sizes in line searches
    ftol : float
      Relative tolerance on function variations
    maxiter : int
      Maximum number of iterations
    callback : callable
      Optional function called after each iteration is complete
    disp : bool
      Print convergence message if True

    Returns
    -------
    x : array
      Gradient descent fix point, local minimizer of f
    """
    x = np.asarray(x0).flatten()
    fval = np.squeeze(f(x))
    it = 0 
    if maxiter == None: 
        maxiter = x.size*1000
    if fprime == None:
        grad_calls, myfprime = _wrap(approx_fprime, (f, step))
    else:
        grad_calls, myfprime = _wrap(fprime, args)

    while it < maxiter:
        it = it + 1
        x0 = x 
        fval0 = fval
        if disp:
            print('Computing gradient...')
        direc = myfprime(x)
        direc = direc / np.sqrt(np.sum(direc**2))
        if disp:
            print('Performing line search...')
        fval, x = _linesearch_brent(f, x, direc, tol=xtol)
        if not callback == None:
            callback(x)
        if (2.0*(fval0-fval) <= ftol*(abs(fval0)+abs(fval))+1e-20): 
            break
        
        if disp:
            print('Number of iterations: %d' % it)
            print('Minimum criterion value: %f' % fval)

    return x 





# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""
Test the empirical null estimator.
"""
import warnings
import numpy as np
from nose.tools import assert_true


from ..empirical_pvalue import (
    NormalEmpiricalNull, smoothed_histogram_from_samples, fdr, fdr_threshold, 
    gaussian_fdr_threshold, gaussian_fdr)

def setup():
    # Suppress warnings during tests to reduce noise
    warnings.simplefilter("ignore")

def teardown():
    # Clear list of warning filters
    warnings.resetwarnings()

def test_efdr():
    # generate the data
    n = 100000
    x = np.random.randn(n)
    x[:3000] += 3
    # make the tests
    efdr = NormalEmpiricalNull(x)
    np.testing.assert_array_less(efdr.fdr(3.0), 0.2)
    np.testing.assert_array_less(-efdr.threshold(alpha=0.05), -2.8)
    np.testing.assert_array_less(-efdr.uncorrected_threshold(alpha=0.001), -2.5)

def test_smooth_histo():
   n = 100
   x = np.random.randn(n)
   h, c = smoothed_histogram_from_samples(x, normalized=True)
   thh = 1. / np.sqrt(2 * np.pi)
   hm = h.max()
   assert_true(np.absolute(hm - thh) < 0.15)

def test_fdr_pos():
    # test with some significant values
    np.random.seed([1])
    x = np.random.rand(100)
    x[:10] *= (.05 / 10)
    q = fdr(x)
    assert_true((q[:10] < .05).all())
    pc = fdr_threshold(x)
    assert_true((pc > .0025) & (pc < .1))

def test_fdr_neg():
    # test without some significant values
    np.random.seed([1])
    x = np.random.rand(100) * .8 + .2
    q =fdr(x)
    assert_true((q > .05).all())
    pc = fdr_threshold(x)
    assert_true(pc == .05 / 100)

def test_gaussian_fdr():
    # Test that fdr works on Gaussian data
    np.random.seed([2])
    x = np.random.randn(100) * 2
    fdr = gaussian_fdr(x)
    assert_true(fdr.min() < .05)
    assert_true(fdr.max() > .99)

def test_gaussian_fdr_threshold():
    np.random.seed([2])
    x = np.random.randn(100) * 2
    ac = gaussian_fdr_threshold(x)
    assert_true(ac > 2.0)
    assert_true(ac < 4.0) 
    assert_true(ac > gaussian_fdr_threshold(x, alpha=.1))

if __name__ == "__main__":
    import nose
    nose.run(argv=['', __file__])



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""
Test functions for models.utils
"""

import numpy as np

from .. import utils

from nose.tools import (assert_equal, assert_true, assert_raises)
from numpy.testing import (assert_array_equal, assert_array_almost_equal)


def test_StepFunction():
    x = np.arange(20)
    y = np.arange(20)
    f = utils.StepFunction(x, y)
    assert_array_almost_equal(f( np.array([[3.2,4.5],[24,-3.1]]) ), [[ 3, 4], [19, 0]])


def test_StepFunctionBadShape():
    x = np.arange(20)
    y = np.arange(21)
    assert_raises(ValueError, utils.StepFunction, x, y)
    x = np.zeros((2, 2))
    y = np.zeros((2, 2))
    assert_raises(ValueError, utils.StepFunction, x, y)



""" Testing the glm module
"""

import numpy as np
from numpy.testing import assert_almost_equal
from nose.tools import assert_true
import numpy.random as nr

from ..mixed_effects_stat import (
    one_sample_ttest, one_sample_ftest, two_sample_ttest, two_sample_ftest, 
    generate_data, t_stat, mfx_stat)


def test_mfx():
    """ Test the generic mixed-effects model"""
    n_samples, n_tests = 20, 100
    np.random.seed(1)
    
    # generate some data
    V1 = np.random.rand(n_samples, n_tests)
    Y = generate_data(np.ones((n_samples, 1)), 0, 1, V1)
    X = np.random.randn(20, 3)

    # compute the test statistics
    t1, = mfx_stat(Y, V1, X, 1,return_t=True, 
                   return_f=False, return_effect=False, 
                   return_var=False)
    assert_true(t1.shape == (n_tests,))
    assert_true(t1.mean() < 5 / np.sqrt(n_tests))
    assert_true((t1.var() < 2) and (t1.var() > .5))
    t2, = mfx_stat(Y, V1, X * np.random.rand(3), 1)
    assert_almost_equal(t1, t2)
    f, = mfx_stat(Y, V1, X, 1, return_t=False, return_f=True)
    assert_almost_equal(t1 ** 2, f)
    v2, = mfx_stat(Y, V1, X, 1, return_t=False, return_var=True)
    assert_true((v2 > 0).all())
    fx, = mfx_stat(Y, V1, X, 1, return_t=False, return_effect=True)
    assert_true(fx.shape == (n_tests,))

def test_t_test():
    """ test that the t test run
    """
    n_samples, n_tests = 15, 100
    data = nr.randn(n_samples, n_tests)
    t = t_stat(data)
    assert_true(t.shape == (n_tests,))
    assert_true( np.abs(t.mean() < 5 / np.sqrt(n_tests)))
    assert_true(t.var() < 2)
    assert_true( t.var() > .5)
    
def test_two_sample_ttest():
    """ test that the mfx ttest indeed runs
    """
    n_samples, n_tests = 15, 4
    np.random.seed(1)
    
    # generate some data
    vardata = np.random.rand(n_samples, n_tests)
    data = generate_data(np.ones(n_samples), 0, 1, vardata)
        
    # compute the test statistics
    u = np.concatenate((np.ones(5), np.zeros(10)))
    t2 = two_sample_ttest(data, vardata, u, n_iter=5)
    assert t2.shape == (n_tests,)
    assert np.abs(t2.mean() < 5 / np.sqrt(n_tests))
    assert t2.var() < 2
    assert t2.var() > .5
    
    # try verbose mode
    t3 = two_sample_ttest(data, vardata, u, n_iter=5, verbose=1)
    assert_almost_equal(t2, t3)

def test_two_sample_ftest():
    """ test that the mfx ttest indeed runs
    """
    n_samples, n_tests = 15, 4
    np.random.seed(1)
    
    # generate some data
    vardata = np.random.rand(n_samples, n_tests)
    data = generate_data(np.ones((n_samples, 1)), 0, 1, vardata)
        
    # compute the test statistics
    u = np.concatenate((np.ones(5), np.zeros(10)))
    t2 = two_sample_ftest(data, vardata, u, n_iter=5)
    assert t2.shape == (n_tests,)
    assert np.abs(t2.mean() < 5 / np.sqrt(n_tests))
    assert t2.var() < 2
    assert t2.var() > .5
    
    # try verbose mode
    t3 = two_sample_ftest(data, vardata, u, n_iter=5, verbose=1)
    assert_almost_equal(t2, t3)

def test_mfx_ttest():
    """ test that the mfx ttest indeed runs
    """
    n_samples, n_tests = 15, 100
    np.random.seed(1)
    
    # generate some data
    vardata = np.random.rand(n_samples, n_tests)
    data = generate_data(np.ones((n_samples, 1)), 0, 1, vardata)
        
    # compute the test statistics
    t2 = one_sample_ttest(data, vardata, n_iter=5)
    assert t2.shape == (n_tests,)
    assert np.abs(t2.mean() < 5 / np.sqrt(n_tests))
    assert t2.var() < 2
    assert t2.var() > .5
    
    # try verbose mode
    t3 = one_sample_ttest(data, vardata, n_iter=5, verbose=1)
    assert_almost_equal(t2, t3)

def test_mfx_ftest():
    """ test that the mfx ftest indeed runs
    """
    n_samples, n_tests = 15, 100
    np.random.seed(1)
    
    # generate some data
    vardata = np.random.rand(n_samples, n_tests)
    data = generate_data(np.ones((n_samples, 1)), 0, 1, vardata)
        
    # compute the test statistics
    f = one_sample_ftest(data, vardata, n_iter=5)
    assert f.shape == (n_tests,)
    assert (np.abs(f.mean() - 1) < 1)
    assert f.var() < 10
    assert f.var() > .2
     

if __name__ == "__main__":
    import nose
    nose.run(argv=['', __file__])




# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""
Edget detection routines
"""
import warnings

import numpy as np
from scipy import ndimage, signal

try:
    # Protect this import as it is compiled code
    from nipy.algorithms.statistics import quantile
except ImportError, e:
    warnings.warn('Could not import fast quantile function: %s' % e)
    quantile = None

                   
################################################################################
# Edge detection

def _fast_abs_percentile(map, percentile=80):
    """ A fast version of the percentile of the absolute value.
    """
    if hasattr(map, 'mask'):
        map = np.asarray(map[np.logical_not(map.mask)])
    map = np.abs(map)
    map = map.ravel()
    if quantile is not None:
        return quantile(map, .01*percentile)
    map.sort()
    nb = map.size
    return map[.01*percentile*nb]


def _orientation_kernel(t):
    """ structure elements for calculating the value of neighbors in several 
        directions
    """ 
    sin = np.sin
    pi  = np.pi
    t = pi*t
    arr =  np.array([[sin(t),        sin(t+.5*pi),  sin(t+pi)    ],
                     [sin(t+1.5*pi), 0,             sin(t+1.5*pi)],
                     [sin(t+pi),     sin(t+.5*pi),  sin(t)       ]])
    return np.round(.5*((1+arr))**2).astype(np.bool)


def _edge_detect(image, high_threshold=.75, low_threshold=.4):
    """ Edge detection for 2D images based on Canny filtering.

        Parameters
        ==========
        image: 2D array
            The image on which edge detection is applied
        high_threshold: float, optional
            The quantile defining the upper threshold of the hysteries 
            thresholding: decrease this to keep more edges
        low_threshold: float, optional
            The quantile defining the lower threshold of the hysteries 
            thresholding: decrease this to extract wider edges

        Returns
        ========
        grad_mag: 2D array of floats
            The magnitude of the gradient
        edge_mask: 2D array of booleans
            A mask of where have edges been detected

        Notes
        ======
        This function is based on a Canny filter, however it has been
        taylored to visualization purposes on brain images: don't use it
        in the general case.

        It computes the norm of the gradient, extracts the ridge by
        keeping only local maximum in each direction, and performs
        hysteresis filtering to keep only edges with high gradient
        magnitude.
    """
    # This code is loosely based on code by Stefan van der Waalt
    # Convert to floats to avoid overflows
    np_err = np.seterr(all='ignore')
    img = signal.wiener(image.astype(np.float))
    np.seterr(**np_err)
    # Where the noise variance is 0, Wiener can create nans
    img[np.isnan(img)] = image[np.isnan(img)]
    img /= img.max()
    grad_x = ndimage.sobel(img, mode='constant', axis=0)
    grad_y = ndimage.sobel(img, mode='constant', axis=1)
    grad_mag = np.sqrt(grad_x**2 + grad_y**2)
    grad_angle = np.arctan2(grad_y, grad_x)
    # Scale the angles in the range [0, 2]
    grad_angle = (grad_angle + np.pi) / np.pi
    # Non-maximal suppression: an edge pixel is only good if its magnitude is
    # greater than its neighbors normal to the edge direction.
    thinner = np.zeros(grad_mag.shape, dtype=np.bool)
    for angle in np.arange(0, 2, .25):
        thinner = thinner | (
                (grad_mag > .85*ndimage.maximum_filter(grad_mag,
                                    footprint=_orientation_kernel(angle))) 
                & (((grad_angle - angle) % 2) < .75)
               )
    # Remove the edges next to the side of the image: they are not reliable
    thinner[0]     = 0
    thinner[-1]    = 0
    thinner[:, 0]  = 0
    thinner[:, -1] = 0

    thinned_grad = thinner * grad_mag
    # Hysteresis thresholding: find seeds above a high threshold, then
    # expand out until we go below the low threshold
    grad_values = thinned_grad[thinner]
    high = thinned_grad > _fast_abs_percentile(grad_values, 100*high_threshold)
    low = thinned_grad >  _fast_abs_percentile(grad_values, 100*low_threshold)
    edge_mask = ndimage.binary_dilation(high, structure=np.ones((3, 3)), 
                                        iterations=-1, mask=low)
    return grad_mag, edge_mask


def _edge_map(image):
    """ Return a maps of edges suitable for visualization.

        Parameters
        ==========
        image: 2D array
            The image that the edges are extracted from.

        Returns
        ========
        edge_mask: 2D masked array
            A mask of the edge as a masked array with parts without
            edges masked and the large extents detected with lower
            coefficients.
    """
    edge_mask = _edge_detect(image)[-1]
    edge_mask = edge_mask.astype(np.float)
    edge_mask = -np.sqrt(ndimage.distance_transform_cdt(edge_mask))
    edge_mask[edge_mask != 0] -= -.05+edge_mask.min()
    edge_mask = np.ma.masked_less(edge_mask, .01)
    return edge_mask





# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""
Parametrized surfaces using a CoordinateMap
"""
import numpy as np

from nose.tools import assert_equal

from nipy.core.api import CoordinateMap, CoordinateSystem
from nipy.core.api import Grid

uv = CoordinateSystem('uv', 'input')
xyz = CoordinateSystem('xyz', 'output')

def parametric_mapping(vals):
    """
    Parametrization of the surface x**2-y**2*z**2+z**3=0
    """
    u = vals[:,0]
    v = vals[:, 1]
    o = np.array([v*(u**2-v**2),
                  u,
                  u**2-v**2]).T
    return o

"""
Let's check that indeed this is a parametrization of that surface
"""

def implicit(vals):
    x = vals[:,0]; y = vals[:,1]; z = vals[:,2]
    return x**2-y**2*z**2+z**3

surface_param = CoordinateMap(uv, xyz, parametric_mapping)

def test_surface():
    assert np.allclose(
        implicit(
            parametric_mapping(
                np.random.standard_normal((40,2))
                )
            ), 
        0)

def test_grid():
    g = Grid(surface_param)
    xyz_grid = g[-1:1:201j,-1:1:101j]
    x, y, z = xyz_grid.transposed_values
    yield assert_equal, x.shape, (201,101)
    yield assert_equal, y.shape, (201,101)
    yield assert_equal, z.shape, (201,101)

def test_grid32():
    # Check that we can use a float32 input and output
    uv32 = CoordinateSystem('uv', 'input', np.float32)
    xyz32 = CoordinateSystem('xyz', 'output', np.float32)
    surface32 = CoordinateMap(uv32, xyz32, parametric_mapping)
    g = Grid(surface32)
    xyz_grid = g[-1:1:201j,-1:1:101j]
    x, y, z = xyz_grid.transposed_values
    yield assert_equal, x.shape, (201,101)
    yield assert_equal, y.shape, (201,101)
    yield assert_equal, z.shape, (201,101)
    yield assert_equal, x.dtype, np.dtype(np.float32)

def test_grid32_c128():
    # Check that we can use a float32 input and complex128 output
    uv32 = CoordinateSystem('uv', 'input', np.float32)
    xyz128 = CoordinateSystem('xyz', 'output', np.complex128)
    def par_c128(x):
        return parametric_mapping(x).astype(np.complex128)
    surface = CoordinateMap(uv32, xyz128, par_c128)
    g = Grid(surface)
    xyz_grid = g[-1:1:201j,-1:1:101j]
    x, y, z = xyz_grid.transposed_values
    yield assert_equal, x.shape, (201,101)
    yield assert_equal, y.shape, (201,101)
    yield assert_equal, z.shape, (201,101)
    yield assert_equal, x.dtype, np.dtype(np.complex128)


def view_surface():
    from enthought.mayavi import mlab
    g = Grid(surface_param)
    xyz_grid = g[-1:1:201j,-1:1:101j]
    x, y, z = xyz_grid.transposed_values
    mlab.mesh(x, y, z)
    mlab.draw()



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""
This module tests nipy's uses of aliased sympy expressions.

That is, sympy.Function's whose value is an arbitrary callable.

We now call these 'implemented functions'.

They've been part of sympy as of 0.7.0

In these tests, the callable's are scipy.interpolate.interp1d instances
representing approximations to Brownian Motions.
"""
import numpy as np

import scipy.interpolate

import sympy

from nipy.fixes.sympy.utilities.lambdify import (implemented_function,
                                                 lambdify)

from nose.tools import assert_true, assert_false, assert_raises

from numpy.testing import (assert_almost_equal, assert_equal,
                           assert_array_almost_equal)


x, y = sympy.symbols(('x', 'y'))


def test_implemented_function():
    # Here we check if the default returned functions are anonymous - in
    # the sense that we can have more than one function with the same name
    f = implemented_function('f', lambda x: 2*x)
    g = implemented_function('f', lambda x: np.sqrt(x))
    l1 = lambdify(x, f(x))
    l2 = lambdify(x, g(x))
    assert_equal(str(f(x)), str(g(x)))
    assert_equal(l1(3), 6)
    assert_equal(l2(3), np.sqrt(3))
    # check that we can pass in a sympy function as input
    func = sympy.Function('myfunc')
    assert_false(hasattr(func, '_imp_'))
    f = implemented_function(func, lambda x: 2*x)
    assert_true(hasattr(func, '_imp_'))


def test_lambdify():
    # Test lambdify with implemented functions
    # first test basic (sympy) lambdify
    f = sympy.cos
    assert_equal(lambdify(x, f(x))(0), 1)
    assert_equal(lambdify(x, 1 + f(x))(0), 2)
    assert_equal(lambdify((x, y), y + f(x))(0, 1), 2)
    # make an implemented function and test
    f = implemented_function("f", lambda x : x+100)
    assert_equal(lambdify(x, f(x))(0), 100)
    assert_equal(lambdify(x, 1 + f(x))(0), 101)
    assert_equal(lambdify((x, y), y + f(x))(0, 1), 101)
    # Error for functions with same name and different implementation
    f2 = implemented_function("f", lambda x : x+101)
    assert_raises(ValueError, lambdify, x, f(f2(x)))
    # our lambdify, like sympy's lambdify, can also handle tuples,
    # lists, dicts as expressions
    lam = lambdify(x, (f(x), x))
    assert_equal(lam(3), (103, 3))
    lam = lambdify(x, [f(x), x])
    assert_equal(lam(3), [103, 3])
    lam = lambdify(x, [f(x), (f(x), x)])
    assert_equal(lam(3), [103, (103, 3)])
    lam = lambdify(x, {f(x): x})
    assert_equal(lam(3), {103: 3})
    lam = lambdify(x, {f(x): x})
    assert_equal(lam(3), {103: 3})
    lam = lambdify(x, {x: f(x)})
    assert_equal(lam(3), {3: 103})


def gen_BrownianMotion():
    X = np.arange(0,5,0.01)
    y = np.random.standard_normal((500,))
    Y = np.cumsum(y)*np.sqrt(0.01)
    B = scipy.interpolate.interp1d(X, Y, bounds_error=0)
    return B


def test_1d():
    B = gen_BrownianMotion()
    Bs = implemented_function("B", B)
    t = sympy.Symbol('t')
    expr = 3*sympy.exp(Bs(t)) + 4
    expected = 3*np.exp(B.y)+4
    ee_vec = lambdify(t, expr, "numpy")
    assert_almost_equal(ee_vec(B.x), expected)
    # with any arbitrary symbol
    b = sympy.Symbol('b')
    expr = 3*sympy.exp(Bs(b)) + 4
    ee_vec = lambdify(b, expr, "numpy")
    assert_almost_equal(ee_vec(B.x), expected)


def test_2d():
    B1, B2 = [gen_BrownianMotion() for _ in range(2)]
    B1s = implemented_function("B1", B1)
    B2s = implemented_function("B2", B2)
    s, t = sympy.symbols(('s', 't'))
    e = B1s(s)+B2s(t)
    ee = lambdify((s,t), e)
    assert_almost_equal(ee(B1.x, B2.x), B1.y + B2.y)



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""
Gaussian Mixture Model Class:
contains the basic fields and methods of GMMs
The class GMM _old uses C bindings which are
computationally and memory efficient.

Author : Bertrand Thirion, 2006-2009
"""

import numpy as np
from scipy.linalg import eigvalsh


class GridDescriptor(object):
    """
    A tiny class to handle cartesian grids
    """

    def __init__(self, dim=1, lim=None, n_bins=None):
        """
        Parameters
        ----------
        dim: int, optional,
             the dimension of the grid
        lim: list of len(2*self.dim),
             the limits of the grid as (xmin, xmax, ymin, ymax, ...)
        n_bins: list of len(self.dim),
             the number of bins in each direction
        """
        self.dim = dim
        if lim is not None:
            self.set(lim, n_bins)
        if np.size(n_bins) == self.dim:
            self.n_bins = np.ravel(np.array(n_bins))

    def set(self, lim, n_bins=10):
        """ set the limits of the grid and the number of bins

        Parameters
        ----------
        lim: list of len(2*self.dim),
             the limits of the grid as (xmin, xmax, ymin, ymax, ...)
        n_bins: list of len(self.dim), optional
             the number of bins in each direction
        """
        if len(lim) == 2 * self.dim:
            self.lim = lim
        else:
            raise ValueError("Wrong dimension for grid definition")
        if np.size(n_bins) == self.dim:
            self.n_bins = np.ravel(np.array(n_bins))
        else:
            raise ValueError("Wrong dimension for grid definition")

    def make_grid(self):
        """ Compute the grid points

        Returns
        -------
        grid: array of shape (nb_nodes, self.dim)
              where nb_nodes is the prod of self.n_bins
        """
        size = np.prod(self.n_bins)
        grid = np.zeros((size, self.dim))
        grange = []

        for j in range(self.dim):
            xm = self.lim[2 * j]
            xM = self.lim[2 * j + 1]
            if np.isscalar(self.n_bins):
                xb = self.n_bins
            else:
                xb = self.n_bins[j]
            gr = xm + float(xM - xm) / (xb - 1) * np.arange(xb).astype('f')
            grange.append(gr)

        if self.dim == 1:
            grid = np.array([[grange[0][i]] for i in range(xb)])

        if self.dim == 2:
            for i in range(self.n_bins[0]):
                for j in range(self.n_bins[1]):
                    grid[i * self.n_bins[1] + j] = np.array(
                        [grange[0][i], grange[1][j]])

        if self.dim == 3:
            for i in range(self.n_bins[0]):
                for j in range(self.n_bins[1]):
                    for k in range(self.n_bins[2]):
                        q = (i * self.n_bins[1] + j) * self.n_bins[2] + k
                        grid[q] = np.array([grange[0][i], grange[1][j],
                                           grange[2][k]])
        if self.dim > 3:
            raise NotImplementedError(
                'only dimensions <4 are currently handled')
        return grid


def best_fitting_GMM(x, krange, prec_type='full', niter=100, delta=1.e-4,
                     ninit=1, verbose=0):
    """
    Given a certain dataset x, find the best-fitting GMM
    with a number k of classes in a certain range defined by krange

    Parameters
    ----------
    x: array of shape (n_samples,dim)
       the data from which the model is estimated
    krange: list of floats,
            the range of values to test for k
    prec_type: string (to be chosen within 'full','diag'), optional,
              the covariance parameterization
    niter: int, optional,
           maximal number of iterations in the estimation process
    delta: float, optional,
           increment of data likelihood at which convergence is declared
    ninit: int
           number of initialization performed
    verbose=0: verbosity mode

    Returns
    -------
    mg : the best-fitting GMM instance
    """
    if np.size(x) == x.shape[0]:
        x = np.reshape(x, (np.size(x), 1))

    dim = x.shape[1]
    bestbic = - np.inf
    for k in krange:
        lgmm = GMM(k, dim, prec_type)
        gmmk = lgmm.initialize_and_estimate(x, None, niter, delta, ninit,
                                            verbose)
        bic = gmmk.evidence(x)
        if bic > bestbic:
            bestbic = bic
            bgmm = gmmk
        if verbose:
            print 'k', k, 'bic', bic
    return bgmm


def plot2D(x, my_gmm, z=None, with_dots=True, log_scale=False, mpaxes=None,
           verbose=0):
    """
    Given a set of points in a plane and a GMM, plot them

    Parameters
    ----------
    x: array of shape (npoints, dim=2),
        sample points
    my_gmm: GMM instance,
            whose density has to be ploted
    z: array of shape (npoints), optional
       that gives a labelling of the points in x
       by default, it is not taken into account
    with_dots, bool, optional
               whether to plot the dots or not
    log_scale: bool, optional
               whether to plot the likelihood in log scale or not
    mpaxes=None, int, optional
                 if not None, axes handle for plotting
    verbose: verbosity mode, optional

    Returns
    -------
    gd, GridDescriptor instance,
        that represents the grid used in the function
    ax, handle to the figure axes

    Notes
    -----
    ``my_gmm`` is assumed to have have a  'nixture_likelihood' method that takes
    an array of points of shape (np, dim) and returns an array of shape
    (np,my_gmm.k) that represents the likelihood component-wise
    """
    import matplotlib.pyplot as plt

    if x.shape[1] != my_gmm.dim:
        raise ValueError('Incompatible dimension between data and model')
    if x.shape[1] != 2:
        raise ValueError('this works only for 2D cases')

    gd1 = GridDescriptor(2)
    xmin, xmax = x.min(0), x.max(0)
    xm = 1.1 * xmin[0] - 0.1 * xmax[0]
    xs = 1.1 * xmax[0] - 0.1 * xmin[0]
    ym = 1.1 * xmin[1] - 0.1 * xmax[1]
    ys = 1.1 * xmax[1] - 0.1 * xmin[1]

    gd1.set([xm, xs, ym, ys], [51, 51])
    grid = gd1.make_grid()
    L = my_gmm.mixture_likelihood(grid)
    if verbose:
        intl = L.sum() * (xs - xm) * (ys - ym) / 2500
        print 'integral of the density on the domain ', intl

    if mpaxes == None:
        plt.figure()
        ax = plt.subplot(1, 1, 1)
    else:
        ax = mpaxes

    gdx = gd1.n_bins[0]
    Pdens = np.reshape(L, (gdx, np.size(L) / gdx))
    extent = [xm, xs, ym, ys]
    if log_scale:
        plt.imshow(np.log(Pdens.T), alpha=2.0, origin='lower',
                  extent=extent)
    else:
        plt.imshow(Pdens.T, alpha=2.0, origin='lower', extent=extent)

    if with_dots:
        if z == None:
            plt.plot(x[:, 0], x[:, 1], 'o')
        else:
            hsv = plt.cm.hsv(range(256))
            col = hsv[range(0, 256, 256 // int(z.max() + 1))]
            for k in range(z.max() + 1):
                plt.plot(x[z == k, 0], x[z == k, 1], 'o', color=col[k])

    plt.axis(extent)
    plt.colorbar()
    return gd1, ax


class GMM(object):
    """Standard GMM.

    this class contains the following members
    k (int): the number of components in the mixture
    dim (int): is the dimension of the data
    prec_type = 'full' (string) is the parameterization
              of the precisions/covariance matrices:
              either 'full' or 'diagonal'.
    means: array of shape (k,dim):
          all the means (mean parameters) of the components
    precisions: array of shape (k,dim,dim):
               the precisions (inverse covariance matrix) of the components
    weights: array of shape(k): weights of the mixture

    fixme
    -----
    no copy method
    """

    def __init__(self, k=1, dim=1, prec_type='full', means=None,
                 precisions=None, weights=None):
        """
        Initialize the structure, at least with the dimensions of the problem

        Parameters
        ----------
        k (int) the number of classes of the model
        dim (int) the dimension of the problem
        prec_type = 'full' : coavriance:precision parameterization
                  (diagonal 'diag' or full 'full').
        means = None: array of shape (self.k,self.dim)
        precisions = None:  array of shape (self.k,self.dim,self.dim)
                   or (self.k, self.dim)
        weights=None: array of shape (self.k)

        By default, means, precision and weights are set as
        zeros()
        eye()
        1/k ones()
        with the correct dimensions
        """
        self.k = k
        self.dim = dim
        self.prec_type = prec_type
        self.means = means
        self.precisions = precisions
        self.weights = weights

        if self.means == None:
            self.means = np.zeros((self.k, self.dim))

        if self.precisions == None:
            if prec_type == 'full':
                prec = np.reshape(np.eye(self.dim), (1, self.dim, self.dim))
                self.precisions = np.repeat(prec, self.k, 0)
            else:
                self.precisions = np.ones((self.k, self.dim))

        if self.weights == None:
            self.weights = np.ones(self.k) * 1.0 / self.k

    def plugin(self, means, precisions, weights):
        """
        Set manually the weights, means and precision of the model

        Parameters
        ----------
        means: array of shape (self.k,self.dim)
        precisions:  array of shape (self.k,self.dim,self.dim)
                     or (self.k, self.dim)
        weights: array of shape (self.k)
        """
        self.means = means
        self.precisions = precisions
        self.weights = weights
        self.check()

    def check(self):
        """
        Checking the shape of different matrices involved in the model
        """
        if self.means.shape[0] != self.k:
            raise ValueError("self.means does not have correct dimensions")

        if self.means.shape[1] != self.dim:
            raise ValueError("self.means does not have correct dimensions")

        if self.weights.size != self.k:
            raise ValueError("self.weights does not have correct dimensions")

        if self.dim != self.precisions.shape[1]:
            raise ValueError(
                "self.precisions does not have correct dimensions")

        if self.prec_type == 'full':
            if self.dim != self.precisions.shape[2]:
                raise ValueError(
                    "self.precisions does not have correct dimensions")

        if self.prec_type == 'diag':
            if np.shape(self.precisions) != np.shape(self.means):
                raise ValueError(
                    "self.precisions does not have correct dimensions")

        if self.precisions.shape[0] != self.k:
            raise ValueError(
                "self.precisions does not have correct dimensions")

        if self.prec_type not in ['full', 'diag']:
            raise ValueError('unknown precisions type')

    def check_x(self, x):
        """
        essentially check that x.shape[1]==self.dim

        x is returned with possibly reshaping
        """
        if np.size(x) == x.shape[0]:
            x = np.reshape(x, (np.size(x), 1))
        if x.shape[1] != self.dim:
            raise ValueError('incorrect size for x')
        return x

    def initialize(self, x):
        """Initializes self according to a certain dataset x:
        1. sets the regularizing hyper-parameters
        2. initializes z using a k-means algorithm, then
        3. upate the parameters

        Parameters
        ----------
        x, array of shape (n_samples,self.dim)
           the data used in the estimation process
        """
        from .utils import kmeans
        n = x.shape[0]

        #1. set the priors
        self.guess_regularizing(x, bcheck=1)

        # 2. initialize the memberships
        if self.k > 1:
            _, z, _ = kmeans(x, self.k)
        else:
            z = np.zeros(n).astype(np.int)

        l = np.zeros((n, self.k))
        l[np.arange(n), z] = 1

        # 3.update the parameters
        self.update(x, l)

    def pop(self, like, tiny=1.e-15):
        """compute the population, i.e. the statistics of allocation

        Parameters
        ----------
        like: array of shape (n_samples,self.k):
              the likelihood of each item being in each class
        """
        sl = np.maximum(tiny, np.sum(like, 1))
        nl = (like.T / sl).T
        return np.sum(nl, 0)

    def update(self, x, l):
        """ Identical to self._Mstep(x,l)
        """
        self._Mstep(x, l)

    def likelihood(self, x):
        """
        return the likelihood of the model for the data x
        the values are weighted by the components weights

        Parameters
        ----------
        x array of shape (n_samples,self.dim)
           the data used in the estimation process

        Returns
        -------
        like, array of shape(n_samples,self.k)
          component-wise likelihood
        """
        like = self.unweighted_likelihood(x)
        like *= self.weights
        return like

    def unweighted_likelihood_(self, x):
        """
        return the likelihood of each data for each component
        the values are not weighted by the component weights

        Parameters
        ----------
        x: array of shape (n_samples,self.dim)
           the data used in the estimation process

        Returns
        -------
        like, array of shape(n_samples,self.k)
          unweighted component-wise likelihood
        """
        n = x.shape[0]
        like = np.zeros((n, self.k))

        for k in range(self.k):
            # compute the data-independent factor first
            w = - np.log(2 * np.pi) * self.dim
            m = np.reshape(self.means[k], (1, self.dim))
            b = self.precisions[k]
            if self.prec_type == 'full':
                w += np.log(eigvalsh(b)).sum()
                dx = m - x
                q = np.sum(np.dot(dx, b) * dx, 1)
            else:
                w += np.sum(np.log(b))
                q = np.dot((m - x) ** 2, b)
            w -= q
            w /= 2
            like[:, k] = np.exp(w)
        return like

    def unweighted_likelihood(self, x):
        """
        return the likelihood of each data for each component
        the values are not weighted by the component weights

        Parameters
        ----------
        x: array of shape (n_samples,self.dim)
           the data used in the estimation process

        Returns
        -------
        like, array of shape(n_samples,self.k)
          unweighted component-wise likelihood

        Notes
        -----
        Hopefully faster
        """
        xt = x.T.copy()
        n = x.shape[0]
        like = np.zeros((n, self.k))

        for k in range(self.k):
            # compute the data-independent factor first
            w = - np.log(2 * np.pi) * self.dim
            m = np.reshape(self.means[k], (self.dim, 1))
            b = self.precisions[k]
            if self.prec_type == 'full':
                w += np.log(eigvalsh(b)).sum()
                dx = xt - m
                sqx = dx * np.dot(b, dx)
                q = np.zeros(n)
                for d in range(self.dim):
                    q += sqx[d]
            else:
                w += np.sum(np.log(b))
                q = np.dot(b, (m - xt) ** 2)
            w -= q
            w /= 2
            like[:, k] = np.exp(w)
        return like

    def mixture_likelihood(self, x):
        """Returns the likelihood of the mixture for x

        Parameters
        ----------
        x: array of shape (n_samples,self.dim)
           the data used in the estimation process
        """
        x = self.check_x(x)
        like = self.likelihood(x)
        sl = np.sum(like, 1)
        return sl

    def average_log_like(self, x, tiny=1.e-15):
        """returns the averaged log-likelihood of the mode for the dataset x

        Parameters
        ----------
        x:  array of shape (n_samples,self.dim)
            the data used in the estimation process
        tiny = 1.e-15: a small constant to avoid numerical singularities
        """
        x = self.check_x(x)
        like = self.likelihood(x)
        sl = np.sum(like, 1)
        sl = np.maximum(sl, tiny)
        return np.mean(np.log(sl))

    def evidence(self, x):
        """Computation of bic approximation of evidence

        Parameters
        ----------
        x array of shape (n_samples,dim)
          the data from which bic is computed

        Returns
        -------
        the bic value
        """
        x = self.check_x(x)
        tiny = 1.e-15
        like = self.likelihood(x)
        return self.bic(like, tiny)

    def bic(self, like, tiny=1.e-15):
        """Computation of bic approximation of evidence

        Parameters
        ----------
        like, array of shape (n_samples, self.k)
           component-wise likelihood
        tiny=1.e-15, a small constant to avoid numerical singularities

        Returns
        -------
        the bic value, float
        """
        sl = np.sum(like, 1)
        sl = np.maximum(sl, tiny)
        bicc = np.sum(np.log(sl))

        # number of parameters
        n = like.shape[0]
        if self.prec_type == 'full':
            eta = self.k * (1 + self.dim + (self.dim * self.dim + 1) / 2) - 1
        else:
            eta = self.k * (1 + 2 * self.dim) - 1
        bicc = bicc - np.log(n) * eta
        return bicc

    def _Estep(self, x):
        """
        E step of the EM algo
        returns the likelihood per class of each data item

        Parameters
        ----------
        x array of shape (n_samples,dim)
          the data used in the estimation process

        Returns
        -------
        likelihood array of shape(n_samples,self.k)
                   component-wise likelihood
        """
        return self.likelihood(x)

    def guess_regularizing(self, x, bcheck=1):
        """
        Set the regularizing priors as weakly informative
        according to Fraley and raftery;
        Journal of Classification 24:155-181 (2007)

        Parameters
        ----------
        x array of shape (n_samples,dim)
          the data used in the estimation process
        """
        small = 0.01
        # the mean of the data
        mx = np.reshape(x.mean(0), (1, self.dim))

        dx = x - mx
        vx = np.dot(dx.T, dx) / x.shape[0]
        if self.prec_type == 'full':
            px = np.reshape(np.diag(1.0 / np.diag(vx)),
                            (1, self.dim, self.dim))
        else:
            px = np.reshape(1.0 / np.diag(vx), (1, self.dim))
        px *= np.exp(2.0 / self.dim * np.log(self.k))
        self.prior_means = np.repeat(mx, self.k, 0)
        self.prior_weights = np.ones(self.k) / self.k
        self.prior_scale = np.repeat(px, self.k, 0)
        self.prior_dof = self.dim + 2
        self.prior_shrinkage = small
        self.weights = np.ones(self.k) * 1.0 / self.k
        if bcheck:
            self.check()

    def _Mstep(self, x, like):
        """
        M step regularized according to the procedure of
        Fraley et al. 2007

        Parameters
        ----------
        x: array of shape(n_samples,self.dim)
           the data from which the model is estimated
        like: array of shape(n_samples,self.k)
           the likelihood of the data under each class
        """
        from numpy.linalg import pinv
        tiny = 1.e-15
        pop = self.pop(like)
        sl = np.maximum(tiny, np.sum(like, 1))
        like = (like.T / sl).T

        # shrinkage,weights,dof
        self.weights = self.prior_weights + pop
        self.weights = self.weights / self.weights.sum()

        # reshape
        pop = np.reshape(pop, (self.k, 1))
        prior_shrinkage = self.prior_shrinkage
        shrinkage = pop + prior_shrinkage

        # means
        means = np.dot(like.T, x) + self.prior_means * prior_shrinkage
        self.means = means / shrinkage

        #precisions
        empmeans = np.dot(like.T, x) / np.maximum(pop, tiny)
        empcov = np.zeros(np.shape(self.precisions))

        if self.prec_type == 'full':
            for k in range(self.k):
                dx = x - empmeans[k]
                empcov[k] = np.dot(dx.T, like[:, k:k + 1] * dx)
            #covariance
            covariance = np.array([pinv(self.prior_scale[k])
                                   for k in range(self.k)])
            covariance += empcov
            dx = np.reshape(empmeans - self.prior_means, (self.k, self.dim, 1))
            addcov = np.array([np.dot(dx[k], dx[k].T) for k in range(self.k)])
            apms = np.reshape(prior_shrinkage * pop / shrinkage,
                              (self.k, 1, 1))
            covariance += (addcov * apms)
            dof = self.prior_dof + pop + self.dim + 2
            covariance /= np.reshape(dof, (self.k, 1, 1))

            # precision
            self.precisions = np.array([pinv(covariance[k]) \
                                       for k in range(self.k)])
        else:
            for k in range(self.k):
                dx = x - empmeans[k]
                empcov[k] = np.sum(dx ** 2 * like[:, k:k + 1], 0)
            # covariance
            covariance = np.array([1.0 / self.prior_scale[k]
                                   for k in range(self.k)])
            covariance += empcov
            dx = np.reshape(empmeans - self.prior_means, (self.k, self.dim, 1))
            addcov = np.array([np.sum(dx[k] ** 2, 0) for k in range(self.k)])
            apms = np.reshape(prior_shrinkage * pop / shrinkage, (self.k, 1))
            covariance += addcov * apms
            dof = self.prior_dof + pop + self.dim + 2
            covariance /= np.reshape(dof, (self.k, 1))

            # precision
            self.precisions = np.array([1.0 / covariance[k] \
                                       for k in range(self.k)])

    def map_label(self, x, like=None):
        """return the MAP labelling of x

        Parameters
        ----------
        x array of shape (n_samples,dim)
          the data under study
        like=None array of shape(n_samples,self.k)
               component-wise likelihood
               if like==None, it is recomputed

        Returns
        -------
        z: array of shape(n_samples): the resulting MAP labelling
           of the rows of x
        """
        if like == None:
            like = self.likelihood(x)
        z = np.argmax(like, 1)
        return z

    def estimate(self, x, niter=100, delta=1.e-4, verbose=0):
        """ Estimation of the model given a dataset x

        Parameters
        ----------
        x array of shape (n_samples,dim)
          the data from which the model is estimated
        niter=100: maximal number of iterations in the estimation process
        delta = 1.e-4: increment of data likelihood at which
              convergence is declared
        verbose=0: verbosity mode

        Returns
        -------
        bic : an asymptotic approximation of model evidence
        """
        # check that the data is OK
        x = self.check_x(x)

        # alternation of E/M step until convergence
        tiny = 1.e-15
        av_ll_old = - np.inf
        for i in range(niter):
            l = self._Estep(x)
            av_ll = np.mean(np.log(np.maximum(np.sum(l, 1), tiny)))
            if av_ll < av_ll_old + delta:
                if verbose:
                    print 'iteration:', i, 'log-likelihood:', av_ll,\
                          'old value:', av_ll_old
                break
            else:
                av_ll_old = av_ll
            if verbose:
                print i, av_ll, self.bic(l)
            self._Mstep(x, l)

        return self.bic(l)

    def initialize_and_estimate(self, x, z=None, niter=100, delta=1.e-4,\
                                ninit=1, verbose=0):
        """Estimation of self given x

        Parameters
        ----------
        x array of shape (n_samples,dim)
          the data from which the model is estimated
        z = None: array of shape (n_samples)
            a prior labelling of the data to initialize the computation
        niter=100: maximal number of iterations in the estimation process
        delta = 1.e-4: increment of data likelihood at which
              convergence is declared
        ninit=1: number of initialization performed
                 to reach a good solution
        verbose=0: verbosity mode

        Returns
        -------
        the best model is returned
        """
        bestbic = - np.inf
        bestgmm = GMM(self.k, self.dim, self.prec_type)
        bestgmm.initialize(x)

        for i in range(ninit):
            # initialization -> Kmeans
            self.initialize(x)

            # alternation of E/M step until convergence
            bic = self.estimate(x, niter=niter, delta=delta, verbose=0)
            if bic > bestbic:
                bestbic = bic
                bestgmm.plugin(self.means, self.precisions, self.weights)

        return bestgmm

    def train(self, x, z=None, niter=100, delta=1.e-4, ninit=1, verbose=0):
        """Idem initialize_and_estimate
        """
        return self.initialize_and_estimate(x, z, niter, delta, ninit, verbose)

    def test(self, x, tiny=1.e-15):
        """Returns the log-likelihood of the mixture for x

        Parameters
        ----------
        x array of shape (n_samples,self.dim)
          the data used in the estimation process

        Returns
        -------
        ll: array of shape(n_samples)
            the log-likelihood of the rows of x
        """
        return np.log(np.maximum(self.mixture_likelihood(x), tiny))

    def show_components(self, x, gd, density=None, mpaxes=None):
        """Function to plot a GMM -- Currently, works only in 1D

        Parameters
        ----------
        x: array of shape(n_samples, dim)
           the data under study
        gd: GridDescriptor instance
        density: array os shape(prod(gd.n_bins))
                 density of the model one the discrete grid implied by gd
                 by default, this is recomputed
        mpaxes: axes handle to make the figure, optional,
                if None, a new figure is created
        """
        import matplotlib.pyplot as plt
        if density is None:
            density = self.mixture_likelihood(gd.make_grid())

        if gd.dim > 1:
            raise NotImplementedError("only implemented in 1D")

        step = 3.5 * np.std(x) / np.exp(np.log(np.size(x)) / 3)
        bins = max(10, int((x.max() - x.min()) / step))
        xmin = 1.1 * x.min() - 0.1 * x.max()
        xmax = 1.1 * x.max() - 0.1 * x.min()
        h, c = np.histogram(x, bins, [xmin, xmax], normed=True)

        # Make code robust to new and old behavior of np.histogram
        c = c[:len(h)]
        offset = (xmax - xmin) / (2 * bins)
        c += offset / 2
        grid = gd.make_grid()

        if mpaxes == None:
            plt.figure()
            ax = plt.axes()
        else:
            ax = mpaxes
        ax.plot(c + offset, h, linewidth=2)

        for k in range(self.k):
            ax.plot(grid, density[:, k], linewidth=2)
        ax.set_title('Fit of the density with a mixture of Gaussians',
                     fontsize=12)

        legend = ['data']
        for k in range(self.k):
            legend.append('component %d' % (k + 1))
        l = ax.legend(tuple(legend))
        for t in l.get_texts():
            t.set_fontsize(12)
        ax.set_xticklabels(ax.get_xticks(), fontsize=12)
        ax.set_yticklabels(ax.get_yticks(), fontsize=12)

    def show(self, x, gd, density=None, axes=None):
        """
        Function to plot a GMM, still in progress
        Currently, works only in 1D and 2D

        Parameters
        ----------
        x: array of shape(n_samples, dim)
           the data under study
        gd: GridDescriptor instance
        density: array os shape(prod(gd.n_bins))
                 density of the model one the discrete grid implied by gd
                 by default, this is recomputed
        """
        import matplotlib.pyplot as plt

        # recompute the density if necessary
        if density is None:
            density = self.mixture_likelihood(gd, x)

        if axes is None:
            axes = plt.figure()

        if gd.dim == 1:
            from ..statistics.empirical_pvalue import \
                smoothed_histogram_from_samples
            h, c = smoothed_histogram_from_samples(x, normalized=True)
            offset = (c.max() - c.min()) / (2 * c.size)
            grid = gd.make_grid()

            h /= h.sum()
            h /= (2 * offset)
            plt.plot(c[: -1] + offset, h)
            plt.plot(grid, density)

        if gd.dim == 2:
            plt.figure()
            xm, xM, ym, yM = gd.lim[0:3]
            gd0 = gd.n_bins[0]
            Pdens = np.reshape(density, (gd0, np.size(density) / gd0))
            axes.imshow(Pdens.T, None, None, None, 'nearest',
                      1.0, None, None, 'lower', [xm, xM, ym, yM])
            axes.plot(x[:, 0], x[:, 1], '.k')
            axes.axis([xm, xM, ym, yM])
        return axes
